import 'dart:convert';

import 'package:crypto/crypto.dart';
import 'package:dio/dio.dart';
import 'package:leoo_junpro/leooflagjunpro/leooappjunpro/leooappconfigjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooappjunpro/leoonetconfigjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leoodaojunpro/leoodaoresultjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooentityjunpro/leoobaseentity.dart';
import 'package:leoo_junpro/leooflagjunpro/leooentityjunpro/leoohostinfoentity.dart';
import 'package:leoo_junpro/leooflagjunpro/leoonetjunpro/leooapphostjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leoonetjunpro/leoohttpmanagerjunpro.dart';
import 'package:sp_util/sp_util.dart';

class LeooHostDaoJunPro {
  static Future<LeooDataResultJunPro> leooFromJunProJsonJunPro(
      Map<String, dynamic> map, String url,
      {bool cancel = false,
      Map<String, dynamic>? headers,
      bool post = true}) async {
    String timestamp = DateTime.now().millisecondsSinceEpoch.toString();
    int uid = SpUtil.getInt(LeooAppConfigJunPro.leooUserIdJunPro)!;
    String sign = md5
        .convert(const Utf8Encoder().convert(
            LeooNetConfigJunPro.leooHostJunProMidJunPro +
                timestamp +
                '$uid' +
                LeooNetConfigJunPro.leooHostJunProSecretKeyJunPro))
        .toString();
    Map<String, dynamic> header = {
      'mid': LeooNetConfigJunPro.leooHostJunProMidJunPro,
      'uid': uid,
      'timestamp': timestamp,
      'sign': sign,
    };
    if (headers != null) {
      header.addAll(headers);
    }
    var res = await httpManager.netFetch(
        url, map, header, Options(method: post ? 'post' : 'get'),
        cancel: cancel);
    if (res.result && res.data is Map) {
      var baseEntity = LeooBaseEntity.fromJson(res.data);
      if (baseEntity.status == 0) {
        return LeooDataResultJunPro(baseEntity.data, true);
      } else {
        return LeooDataResultJunPro(baseEntity.data, false);
      }
    }
    return LeooDataResultJunPro(res, false, dataError: true);
  }

  static cancel() {
    httpManager.cancel();
  }

  static Future<LeooDataResultJunPro> leooHostListJunPro(int pageNum) async {
    Map<String, dynamic> other = {
      'pageNum': pageNum,
      'query': {'regionCode': 'all'}
    };
    var res = await leooFromJunProJsonJunPro(
        other, LeooAppHostJunPro.leooHostListJunPro());
    if (res.result) {
      var data = (res.data['records'] as List)
          .map((v) => LeooHostInfoEntity.fromJson(v))
          .toList();
      return LeooDataResultJunPro(data, true);
    }
    return res;
  }

  static Future<LeooDataResultJunPro> leooHostInfoJunPro(int? relateUid) async {
    Map<String, dynamic> other = {'relateUid': relateUid};
    var res = await leooFromJunProJsonJunPro(
        other, LeooAppHostJunPro.leooHostInfoJunPro());
    if (res.result) {
      var entity = LeooHostInfoEntity.fromJson(res.data);
      return LeooDataResultJunPro(entity, true);
    }
    return res;
  }

  static Future<LeooDataResultJunPro> leooVideoCallJunPro(String? roomId,
      String? agoId, int? remoteUid, int? remoteId, String? remoteToken) async {
    Map<String, dynamic> other = {
      'roomId': roomId,
      'agoId': agoId,
      'remoteUid': remoteUid,
      'remoteId': remoteId,
      'remoteToken': remoteToken
    };
    var res = await leooFromJunProJsonJunPro(
        other, LeooAppHostJunPro.leooVideoCallJunPro());
    if (res.result) {
      return LeooDataResultJunPro(res, true);
    }
    return res;
  }

  static Future<LeooDataResultJunPro> leooVideoCloseJunPro(
    String? roomId,
    int? remoteUid,
  ) async {
    Map<String, dynamic> other = {'roomId': roomId, 'remoteUid': remoteUid};
    var res = await leooFromJunProJsonJunPro(
        other, LeooAppHostJunPro.leooVideoCloseJunPro());
    if (res.result) {
      return LeooDataResultJunPro(res, true);
    }
    return res;
  }
}
