import 'dart:async';

import 'package:leoo_junpro/leooflagjunpro/leoodbjunpro/leoosqlproviderjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooentityjunpro/leootranslatorentity.dart';
import 'package:sqflite/sqflite.dart';

///翻译文本表
class LeooTranslatorDbProviderJunPro extends LeooBaseDbProviderJunPro {
  final String name = 'TranslatorInfo';

  final String columnId = 'id';
  final String columnMsgId = 'msgId';
  final String columnUserId = 'userId';
  final String columnText = 'text';
  final String columnData = 'data';

  LeooTranslatorDbProviderJunPro();

  Map<String, dynamic> toMap(
      String msgId, int? userId, String text, String data) {
    Map<String, dynamic> map = {
      columnMsgId: msgId,
      columnUserId: userId,
      columnText: text,
      columnData: data
    };
    return map;
  }

  @override
  tableSqlString() {
    return tableBaseString(name, columnId) +
        '''
        $columnMsgId text not null,
        $columnUserId int,
        $columnText text not null,
        $columnData text not null)
      ''';
  }

  @override
  tableName() {
    return name;
  }

  Future<Map<String, dynamic>?> _getItemProvider(
      Database db, String msgId, int? userId) async {
    List<Map<String, dynamic>> maps = await db.query(name,
        columns: [columnId, columnMsgId, columnUserId, columnText, columnData],
        where: '$columnMsgId = ? and $columnUserId = ?',
        whereArgs: [msgId, userId]);
    if (maps.isNotEmpty) {
      return maps.first;
    }
    return null;
  }

  ///插入到数据库
  Future insert(String msgId, int? userId, String text, String? data) async {
    Database db = await getDataBase();
    var translateProvider = await _getItemProvider(db, msgId, userId);
    if (translateProvider != null) {
      await db.update(name, toMap(msgId, userId, text, data ?? 'null'),
          where: '$columnMsgId = ? and $columnUserId = ?',
          whereArgs: [msgId, userId]);
    }
    return await db.insert(name, toMap(msgId, userId, text, data ?? 'null'));
  }

  ///获取事件数据
  Future<Map<String?, String?>> getTranslatorDataList(int? userId) async {
    Database db = await getDataBase();
    List<Map<String, dynamic>> maps = await db.query(name,
        columns: [columnId, columnMsgId, columnUserId, columnText, columnData],
        where: '$columnUserId = ?',
        whereArgs: [userId]);
    Map<String?, String?> translatorMap = {};
    for (var map in maps) {
      var entity = LeooTranslatorEntity.fromJson(map);
      if (entity.data == 'null') {
        translatorMap[entity.msgId] = null;
      } else {
        translatorMap[entity.msgId] = entity.data;
      }
    }
    return translatorMap;
  }
}
