import 'dart:async';

import 'package:flutter/foundation.dart';
import 'package:leoo_junpro/leooflagjunpro/leoodbjunpro/leoosqlproviderjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooentityjunpro/leoouserentity.dart';
import 'package:leoo_junpro/leooflagjunpro/leooutilsjunpro/leoocodeutilsjunpro.dart';
import 'package:sqflite/sqflite.dart';

///用户表
class LeooUserInfoDbProviderJunPro extends LeooBaseDbProviderJunPro {
  final String name = 'UserInfoDb';

  final String columnId = '_id';
  final String columnUserId = 'userId';
  final String columnData = 'data';

  int? userId;
  String? data;

  LeooUserInfoDbProviderJunPro();

  Map<String, dynamic> toMap(int? userId, String data) {
    Map<String, dynamic> map = {columnUserId: userId, columnData: data};
    return map;
  }

  LeooUserInfoDbProviderJunPro.fromMap(Map map) {
    userId = map[columnUserId];
    data = map[columnData];
  }

  @override
  tableSqlString() {
    return tableBaseString(name, columnId) +
        '''
        $columnUserId int,
        $columnData text not null)
      ''';
  }

  @override
  tableName() {
    return name;
  }

  Future _getUserProvider(Database db, int? userId) async {
    List<Map<String, dynamic>> maps = await db.query(name,
        columns: [columnId, columnUserId, columnData],
        where: '$columnUserId = ?',
        whereArgs: [userId]);
    if (maps.isNotEmpty) {
      LeooUserInfoDbProviderJunPro userProvider =
          LeooUserInfoDbProviderJunPro.fromMap(maps.first);
      return userProvider;
    }
    return null;
  }

  ///插入到数据库
  Future insert(int? userId, String eventMapString) async {
    Database db = await getDataBase();
    var userProvider = await _getUserProvider(db, userId);
    if (userProvider != null) {
      return await db.update(name, toMap(userId, eventMapString),
          where: '$columnUserId = ?', whereArgs: [userId]);
    }
    return await db.insert(name, toMap(userId, eventMapString));
  }

  ///获取事件数据
  Future<LeooUserEntity?> getUserInfo(int? userId) async {
    Database db = await getDataBase();
    var userProvider = await _getUserProvider(db, userId);
    if (userProvider != null) {
      ///使用 compute 的 Isolate 优化 json decode
      var mapData = await compute(LeooCodeUtilsJunPro.leooDecodeMapResultJunPro,
          userProvider.data as String);
      return LeooUserEntity.fromJson(mapData);
    }
    return null;
  }
}
