import 'dart:convert';
import 'dart:io';

import 'package:date_format/date_format.dart';
import 'package:flutter/material.dart';
import 'package:im_flutter_sdk/im_flutter_sdk.dart';
import 'package:leoo_junpro/generated/l10n.dart' as intl;
import 'package:leoo_junpro/leooflagjunpro/leooappjunpro/leooappconfigjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooappjunpro/leoonetconfigjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leoostringjunpro/leoostylejunpro.dart';
import 'package:path_provider/path_provider.dart';
import 'package:permission_handler/permission_handler.dart';
import 'package:sp_util/sp_util.dart';
import 'package:translator/translator.dart';

///isolate 的 compute 需要静态方法
class LeooCodeUtilsJunPro {
  static dynamic leooDecodeResultJunPro(String data) {
    return jsonDecode(data);
  }

  static List<dynamic> leooDecodeListResultJunPro(String data) {
    return jsonDecode(data);
  }

  static Map<String, dynamic> leooDecodeMapResultJunPro(String data) {
    return jsonDecode(data);
  }

  static String leooGetImageFileJunPro(String? url) {
    if (url?.startsWith('file://') == true) {
      return url!.replaceAll('file://', '');
    } else {
      return (url ?? '');
    }
  }

  static Future<Directory?> leooGetLocalPathJunPro() async {
    Directory? appDir;
    if (Platform.isIOS) {
      appDir = await getApplicationDocumentsDirectory();
    } else {
      appDir = await getExternalStorageDirectory();
      PermissionStatus status = await Permission.storage.status;
      if (status == PermissionStatus.denied) {
        status = await Permission.storage.request();
        if (status == PermissionStatus.denied) {
          return null;
        }
      }
    }
    String appDocPath =
        appDir!.path + '/' + LeooNetConfigJunPro.leooPkgNameJunPro;
    Directory appPath = Directory(appDocPath);
    await appPath.create(recursive: true);
    return appPath;
  }

  static String leooCompressImageUrlJunPro(String url, {int width = 0}) {
    try {
      if (width != 0) {
        var spacing = url.contains('?') ? '&' : '?';
        return '$url${spacing}x-oss-process=image/resize,w_${width * 3}';
      }
    } catch (e) {
      LeooNetConfigJunPro.leooPrintLogJunPro(e);
    }
    return url;
  }

  static String leooGetTimeHNStrJunPro(DateTime date) {
    return formatDate(date, [HH, ':', nn]);
  }

  static String leooGetTimeStrJunPro(DateTime date,
      {bool min = false, bool seconds = false}) {
    if (seconds) {
      return formatDate(date, [ss]);
    } else if (min) {
      return formatDate(date, [nn, ':', ss]);
    } else {
      return formatDate(date, [HH, ':', nn, ':', ss]);
    }
  }

  static String leooGetTimeDayStrJunPro(String time, {bool min = true}) {
    if (time.isEmpty == false) {
      if (min) {
        return formatDate(DateTime.fromMillisecondsSinceEpoch(int.parse(time)),
            [yyyy, '-', mm, '-', dd, ' ', HH, ':', nn, ':', ss]);
      } else {
        return formatDate(DateTime.fromMillisecondsSinceEpoch(int.parse(time)),
            [yyyy, '-', mm, '-', dd]);
      }
    }
    return '';
  }

  static String leooGetTimeDayStr2JunPro(String time, {bool year = false}) {
    if (time.isEmpty == false) {
      if (year) {
        return formatDate(DateTime.parse(time), [yyyy, '-', mm, '-', dd]);
      } else {
        return formatDate(DateTime.parse(time), [mm, '月', dd, '日']);
      }
    }
    return '';
  }

  static const double leooMillisLimitJunPro = 1000.0;
  static const double leooSecondsLimitJunPro = 60 * leooMillisLimitJunPro;
  static const double leooMinutesLimitJunPro = 60 * leooSecondsLimitJunPro;
  static const double leooHoursLimitJunPro = 24 * leooMinutesLimitJunPro;
  static const double leooDaysLimitJunPro = 30 * leooHoursLimitJunPro;

  ///日期格式转换
  static String leooGetNewsTimeStrJunPro(context, DateTime date) {
    if (date.millisecondsSinceEpoch == 0) {
      return '';
    }
    int subTimes =
        DateTime.now().millisecondsSinceEpoch - date.millisecondsSinceEpoch;
    if (subTimes < leooMillisLimitJunPro) {
      return intl.S.of(context).timeNow;
    } else if (subTimes < leooSecondsLimitJunPro) {
      return intl.S
          .of(context)
          .timeSeconds((subTimes / leooMillisLimitJunPro).round().toString());
    } else if (subTimes < leooMinutesLimitJunPro) {
      return intl.S
          .of(context)
          .timeMin((subTimes / leooSecondsLimitJunPro).round().toString());
    } else if (subTimes < leooHoursLimitJunPro) {
      return intl.S
          .of(context)
          .timeHours((subTimes / leooMinutesLimitJunPro).round().toString());
    } else if (subTimes < leooDaysLimitJunPro) {
      return intl.S
          .of(context)
          .timeDays((subTimes / leooHoursLimitJunPro).round().toString());
    } else {
      return formatDate(date, [yyyy, '-', mm, '-', dd]);
    }
  }

  ///是否需要显示时间，相差2分钟
  static bool leooNeedShowTimeJunPro(int sentTime1, int sentTime2) {
    return (sentTime1 - sentTime2).abs() > 2 * 60 * 1000;
  }

  static String leooCompareStringJunPro(String? str) {
    return str?.length == 1 ? '0$str' : (str ?? '');
  }

  static Future<String?> leooTranslatorMessageTextJunPro(String text,
      {bool send = false}) async {
    try {
      final translator = GoogleTranslator();
      if (send) {
        var translatedText = await translator.translate(text);
        return translatedText.text;
      } else {
        var translatedText = await translator.translate(text,
            to: intl
                .S
                .delegate
                .supportedLocales[
                    SpUtil.getInt(LeooAppConfigJunPro.leooLocaleJunPro)!]
                .languageCode);
        return translatedText.text;
      }
    } catch (e) {
      LeooNetConfigJunPro.leooPrintLogJunPro(e);
    }
    return null;
  }

  static bool leooNeedShowTimeListJunPro(int index, List allList) {
    bool needShow = false;
    //消息是逆序的
    if (index == allList.length - 1) {
      //第一条消息一定显示时间
      needShow = true;
    } else {
      //如果满足条件，则显示时间
      var lastMessage = allList[index + 1];
      var curMessage = allList[index];
      if (LeooCodeUtilsJunPro.leooNeedShowTimeJunPro(
          lastMessage.localTime!, curMessage.localTime!)) {
        needShow = true;
      }
    }
    return needShow;
  }

  static String? leooGetDigestJunPro(
      EMMessageBody? content, BuildContext context) {
    if (content is EMTextMessageBody) {
      return content.content;
    } else if (content is EMImageMessageBody ||
        content is EMCustomMessageBody &&
            content.event == LeooAppConfigJunPro.leooImageEventJunPro) {
      return '[${intl.S.of(context).messageImage}]';
    } else if (content is EMCustomMessageBody &&
        content.event == LeooAppConfigJunPro.leooGiftEventJunPro) {
      return '[${content.params?['name']}]';
    } else {
      return '';
    }
  }

  static String leooGetDetailTypeJunPro(String? bizType) {
    switch (bizType) {
      case 'TALKING':
        return LeooIconsJunPro.leooTypeCallJunPro;
      case 'LITTLE_VIDEO':
        return LeooIconsJunPro.leooTypeVideoJunPro;
      case 'WITHDRAW_REQUEST':
        return LeooIconsJunPro.leooTypeDrawJunPro;
      case 'WITHDRAW_BACK':
        return LeooIconsJunPro.leooTypeDrawJunPro;
      case 'MESSAGE':
        return LeooIconsJunPro.leooTypeMessageJunPro;
      case 'GIFT':
        return LeooIconsJunPro.leooTypeGiftJunPro;
      case 'MATCHING':
        return LeooIconsJunPro.leooTypeCallJunPro;
      default:
        return LeooIconsJunPro.leooTypeMessageJunPro;
    }
  }

  ///取小数点后几位
  // @param num 数值
  // @param location 几位
  static String leooFormatNumJunPro(double num, int location) {
    if ((num.toString().length - num.toString().lastIndexOf(".") - 1) <
        location) {
      //小数点后有几位小数
      return num.toStringAsFixed(location)
          .substring(0, num.toString().lastIndexOf(".") + location + 1)
          .toString();
    } else {
      return num.toString()
          .substring(0, num.toString().lastIndexOf(".") + location + 1)
          .toString();
    }
  }

  static void onSetState(State state) {
    try {
      if (!state.mounted) {
        return;
      }
      state.setState(() {});
    } catch (e) {
      LeooNetConfigJunPro.leooPrintLogJunPro(e);
    }
  }
}
