import 'dart:async';
import 'dart:io';

import 'package:connectivity/connectivity.dart';
import 'package:extended_image/extended_image.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:flutter_spinkit/flutter_spinkit.dart';
import 'package:flutter_statusbarcolor_ns/flutter_statusbarcolor_ns.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:leoo_junpro/generated/l10n.dart';
import 'package:leoo_junpro/leooflagjunpro/leooappjunpro/leoonetconfigjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooentityjunpro/leoobaseentity.dart';
import 'package:leoo_junpro/leooflagjunpro/leoonetjunpro/leooresultdatajunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leoostringjunpro/leoostylejunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooutilsjunpro/leoocodeutilsjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooutilsjunpro/leoomediautilsjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooutilsjunpro/leoonavigatorutilsjunpro.dart';
import 'package:leoo_junpro/leoorouterjunpro.dart';
import 'package:lottie/lottie.dart';
import 'package:move_to_background/move_to_background.dart';
import 'package:url_launcher/url_launcher.dart';

///通用逻辑-基本方法管理
class LeooCommonUtilsJunPro {
  static void leooShowCustomToastJunPro(String msg, [bool vibrate = false]) {
    LeooNetConfigJunPro.leooPrintLogJunPro('showCustomToast: $msg');
    try {
      if (vibrate) {
        LeooMediaUtilsJunPro.instance.startVibration(repeat: -1);
      }
      FToast fToast = FToast();
      fToast.init(LeooRouterJunPro.leooContextJunPro);
      fToast.showToast(
          child: Container(
            padding: EdgeInsets.all(20.w),
            decoration: ShapeDecoration(
              color: LeooColorsJunPro.leooPopBgJunPro,
              shape: RoundedRectangleBorder(
                borderRadius: BorderRadius.all(Radius.circular(20)),
              ),
            ),
            child: Text(msg,
                style:
                    LeooTextJunPro.leooGetTextStyleJunPro(16, FontWeight.bold)),
          ),
          gravity: ToastGravity.TOP,
          toastDuration: const Duration(seconds: 2));
    } catch (e) {
      LeooNetConfigJunPro.leooPrintLogJunPro(e);
    }
  }

  /// 错误提示统一
  static leooShowErrorToastJunPro([res]) async {
    try {
      var connectivityResult = await (Connectivity().checkConnectivity());
      if (connectivityResult == ConnectivityResult.none) {
        LeooCommonUtilsJunPro.leooShowCustomToastJunPro(
            S.of(LeooRouterJunPro.leooContextJunPro).networkError);
      } else {
        if (res == null || res.data is LeooResultDataJunPro) {
          LeooCommonUtilsJunPro.leooShowCustomToastJunPro(
              S.of(LeooRouterJunPro.leooContextJunPro).appHttpErr);
        } else if (res is LeooBaseEntity) {
          if (res.msg.isEmpty == false) {
            LeooCommonUtilsJunPro.leooShowCustomToastJunPro(res.msg);
          } else if (res.message.isEmpty == false) {
            LeooCommonUtilsJunPro.leooShowCustomToastJunPro(res.message);
          } else {
            LeooCommonUtilsJunPro.leooShowCustomToastJunPro(
                S.of(LeooRouterJunPro.leooContextJunPro).appHttpErr);
          }
        } else if (res.data is LeooBaseEntity) {
          if (res.data.msg.isEmpty == false) {
            LeooCommonUtilsJunPro.leooShowCustomToastJunPro(res.data.msg);
          } else if (res.data.message.isEmpty == false) {
            LeooCommonUtilsJunPro.leooShowCustomToastJunPro(res.data.message);
          } else {
            LeooCommonUtilsJunPro.leooShowCustomToastJunPro(
                S.of(LeooRouterJunPro.leooContextJunPro).appHttpErr);
          }
        } else {
          LeooCommonUtilsJunPro.leooShowCustomToastJunPro(
              S.of(LeooRouterJunPro.leooContextJunPro).appHttpErr);
        }
      }
    } catch (e) {
      LeooNetConfigJunPro.leooPrintLogJunPro(e);
    }
  }

  static leooCopyJunPro(BuildContext context, String data) {
    Clipboard.setData(ClipboardData(text: data));
    leooShowCustomToastJunPro(S.of(context).copySuccess);
  }

  static leooLaunchOutURLJunPro(BuildContext context, String url,
      {bool email = false}) async {
    if (await canLaunch(url)) {
      if (email) {
        await launch(url);
      } else if (await canLaunch(url.replaceAll(' ', ''))) {
        await launch(url.replaceAll(' ', ''));
      } else if (await canLaunch(url.replaceAll('//', ''))) {
        await launch(url.replaceAll('//', ''));
      } else {
        await launch(url);
      }
    } else {
      leooShowCustomToastJunPro(S.of(context).launcherError(url));
    }
  }

  static int leooPageStatusSuccessJunPro = 0;
  static int leooPageStatusLoadingJunPro = 1;
  static int leooPageStatusErrorJunPro = 2;
  static int leooPageStatusEmptyJunPro = 3;
  static int leooPageStatusProgressJunPro = 4;

  static Widget leooCheckPageStatusJunPro(BuildContext context, status,
      {VoidCallback? onTap,
      String? empty,
      String? emptyIcon,
      Widget? createWidget}) {
    if (status == leooPageStatusProgressJunPro) {
      ///进度条加载中
      return SizedBox(
          height: 2.w,
          child: LinearProgressIndicator(
            backgroundColor: LeooColorsJunPro.leooTextHintJunPro,
            valueColor: AlwaysStoppedAnimation<Color>(
              LeooColorsJunPro.leooAppJunPro,
            ),
          ));
    } else if (status == leooPageStatusLoadingJunPro) {
      ///默认加载中
      return Container(
        width: 1.sw,
        padding: EdgeInsets.only(left: 60.w, right: 60.w),
        child: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            Expanded(child: Container()),
            Container(
              width: 120.w,
              height: 120.w,
              padding: EdgeInsets.all(10.w),
              child: SpinKitFadingCircle(
                color: LeooColorsJunPro.leooAppJunPro,
                size: 40.w,
              ),
            ),
            Text(S.of(context).loading,
                style:
                    LeooTextJunPro.leooGetTextStyleJunPro(14, FontWeight.bold)),
            Expanded(flex: 2, child: Container()),
          ],
        ),
      );
    } else {
      return InkWell(
        child: Container(
          width: 1.sw,
          padding: EdgeInsets.only(left: 60.w, right: 60.w),
          child: Column(
            mainAxisAlignment: MainAxisAlignment.center,
            children: [
              Expanded(child: Container()),
              LeooCommonUtilsJunPro.leooGetLoadWidgetJunPro(
                  status == leooPageStatusEmptyJunPro
                      ? emptyIcon ?? LeooIconsJunPro.leooEmptyDataJunPro
                      : LeooIconsJunPro.leooEmptyNetworkJunPro,
                  width: 120.w,
                  height: 120.w),
              SizedBox(height: 15.w),
              Text(
                status == leooPageStatusEmptyJunPro
                    ? empty ?? S.of(context).emptyList
                    : S.of(context).netWork,
                style:
                    LeooTextJunPro.leooGetTextStyleJunPro(14, FontWeight.bold),
                textAlign: TextAlign.center,
              ),
              if (status == leooPageStatusEmptyJunPro && createWidget != null)
                createWidget,
              Expanded(flex: 2, child: Container()),
            ],
          ),
        ),
        onTap: LeooNavigatorUtilsJunPro.leooDebounceJunPro(() => onTap?.call()),
      );
    }
  }

  static Widget leooGetLoadWidgetJunPro(String? url,
      {double? width,
      double? height,
      Alignment? alignment,
      bool color = true,
      BoxFit? fit,
      Map<String, String>? headers,
      String? load,
      String? imageStartUrl,
      String? package = LeooIconsJunPro.leooPackageJunPro}) {
    if (url?.isEmpty == false) {
      if (url!.startsWith(LeooIconsJunPro.leooImageAppJunPro) ||
          url.startsWith(imageStartUrl ?? '-') ||
          url.endsWith(LeooIconsJunPro.leooAppInEndJunPro)) {
        return Image.asset(
          url,
          width: width,
          height: height,
          fit: fit,
          package:
              url.endsWith(LeooIconsJunPro.leooAppInEndJunPro) ? null : package,
        );
      } else if (!url.startsWith('http')) {
        return ExtendedImage.file(
          File(url),
          width: width,
          height: height,
          fit: fit ?? BoxFit.cover,
          alignment: alignment ?? Alignment.topCenter,
          enableLoadState: true,
          loadStateChanged: ((ExtendedImageState state) {
            switch (state.extendedImageLoadState) {
              case LoadState.loading:
                if (state.loadingProgress == null) {
                  return Image.asset(
                    load ?? LeooIconsJunPro.leooLoadPicJunPro,
                    width: width,
                    height: height,
                    fit: fit ?? BoxFit.cover,
                    alignment: alignment ?? Alignment.center,
                    package: load == null
                        ? LeooIconsJunPro.leooPackageJunPro
                        : package,
                  );
                }
                return Container(
                  color: color
                      ? LeooColorsJunPro.leooLineJunPro
                      : Colors.transparent,
                  child: Center(
                    child: CircularProgressIndicator(
                        valueColor: AlwaysStoppedAnimation<Color>(
                            LeooColorsJunPro.leooTextHintJunPro),
                        value: state.loadingProgress!.expectedTotalBytes != null
                            ? state.loadingProgress!.cumulativeBytesLoaded /
                                state.loadingProgress!.expectedTotalBytes!
                            : null),
                  ),
                );
              case LoadState.completed:
                return state.completedWidget;
              default:
                return Image.asset(
                  load ?? LeooIconsJunPro.leooLoadPicJunPro,
                  width: width,
                  height: height,
                  fit: fit ?? BoxFit.cover,
                  alignment: alignment ?? Alignment.center,
                  package: load == null
                      ? LeooIconsJunPro.leooPackageJunPro
                      : package,
                );
            }
          }),
        );
      } else {
        return ExtendedImage.network(
          !url.startsWith(LeooIconsJunPro.leooImageUrlJunPro) &&
                  headers == null &&
                  (width ?? 0) != 0
              ? LeooCodeUtilsJunPro.leooCompressImageUrlJunPro(url,
                  width: width!.toInt())
              : url,
          width: width,
          height: height,
          fit: fit ??
              (url.startsWith(LeooIconsJunPro.leooImageUrlJunPro) == true
                  ? BoxFit.contain
                  : BoxFit.cover),
          headers: headers,
          alignment: alignment ?? Alignment.topCenter,
          enableLoadState: true,
          handleLoadingProgress: true,
          loadStateChanged: ((ExtendedImageState state) {
            switch (state.extendedImageLoadState) {
              case LoadState.loading:
                if (state.loadingProgress == null) {
                  return Image.asset(
                    load ?? LeooIconsJunPro.leooLoadPicJunPro,
                    width: width,
                    height: height,
                    fit: fit ?? BoxFit.cover,
                    alignment: alignment ?? Alignment.center,
                    package: load == null
                        ? LeooIconsJunPro.leooPackageJunPro
                        : package,
                  );
                }
                return Container(
                  color: color
                      ? LeooColorsJunPro.leooLineJunPro
                      : Colors.transparent,
                  child: Center(
                    child: CircularProgressIndicator(
                        valueColor: AlwaysStoppedAnimation<Color>(
                            LeooColorsJunPro.leooTextHintJunPro),
                        value: state.loadingProgress!.expectedTotalBytes != null
                            ? state.loadingProgress!.cumulativeBytesLoaded /
                                state.loadingProgress!.expectedTotalBytes!
                            : null),
                  ),
                );
              case LoadState.completed:
                return state.completedWidget;
              default:
                return Image.asset(
                  load ?? LeooIconsJunPro.leooLoadPicJunPro,
                  width: width,
                  height: height,
                  fit: fit ?? BoxFit.cover,
                  alignment: alignment ?? Alignment.center,
                  package: load == null
                      ? LeooIconsJunPro.leooPackageJunPro
                      : package,
                );
            }
          }),
        );
      }
    } else {
      return Image.asset(
        load ?? LeooIconsJunPro.leooLoadPicJunPro,
        width: width,
        height: height,
        fit: fit ?? BoxFit.cover,
        alignment: alignment ?? Alignment.center,
        package: load == null ? LeooIconsJunPro.leooPackageJunPro : package,
      );
    }
  }

  //lottie加载本地还是网络
  static Widget leooLottieJunPro(String url, {double? width, double? height}) {
    if (url.startsWith('http')) {
      return Lottie.network(
        url,
        width: width,
        height: height,
      );
    } else {
      return Lottie.asset(url,
          width: width,
          height: height,
          package: LeooIconsJunPro.leooPackageJunPro);
    }
  }

  /// 不退出
  static Future<bool> leooDialogExitAppJunPro(BuildContext context) async {
    MoveToBackground.moveTaskToBack();
    return Future.value(false);
  }

  static leooCheckIsCurrentJunPro(context, back, [noBack]) async {
    return await Future(() {
      if (ModalRoute.of(context)?.isCurrent == true) {
        return back();
      } else {
        return noBack?.call();
      }
    });
  }

  static leooCheckThemeModeJunPro(ThemeMode themeMode) async {
    Future(() {
      try {
        if (themeMode == ThemeMode.dark) {
          FlutterStatusbarcolor.setStatusBarWhiteForeground(false);
          FlutterStatusbarcolor.setNavigationBarWhiteForeground(false);
        } else {}
      } catch (e) {
        LeooNetConfigJunPro.leooPrintLogJunPro(e);
      }
    });
  }
}
