import 'dart:core';
import 'dart:io';

import 'package:firebase_analytics/firebase_analytics.dart';
import 'package:firebase_messaging/firebase_messaging.dart';
import 'package:flutter/material.dart';
import 'package:im_flutter_sdk/im_flutter_sdk.dart';
import 'package:leoo_junpro/generated/l10n.dart';
import 'package:leoo_junpro/leooflagjunpro/leooappjunpro/leooappconfigjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooappjunpro/leoonetconfigjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooeventjunpro/leooindexjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooeventjunpro/leoomaineventjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooutilsjunpro/leoocommonutilsjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooutilsjunpro/leoonavigatorutilsjunpro.dart';
import 'package:leoo_junpro/leoorouterjunpro.dart';
import 'package:sp_util/sp_util.dart';

//定义一个抽象类-聊天监听，视频通话逻辑
abstract class OnImPluginDelegate {
  void onMessageReceived(List<EMMessage> messages);

  void onConnectionStatusChange(bool connected);
}

///im
class LeooImUtilsJunPro with EMConnectionListener, EMChatManagerListener {
  factory LeooImUtilsJunPro() => _getInstance()!;

  static LeooImUtilsJunPro get instance => _getInstance()!;
  static LeooImUtilsJunPro? _instance;

  LeooImUtilsJunPro._internal();

  static LeooImUtilsJunPro? _getInstance() {
    _instance ??= LeooImUtilsJunPro._internal();
    return _instance;
  }

  List<OnImPluginDelegate> imPluginDelegateList = [];

  bool isReception = true;
  bool clearConversation = false;
  bool delete = false;
  bool updateOwnUserInfo = false;

  void initLoneFirebaseLy() {
    try {
      if (Platform.isIOS) {
        FirebaseMessaging.instance.getAPNSToken().then((token) {
          LeooNetConfigJunPro.leooPrintLogJunPro(
              'FirebaseMessaging getAPNSToken: $token');
          if (token?.isEmpty == false) {
            EMClient.getInstance.pushManager.updateAPNsDeviceToken(token!);
          }
        });
      }
      FirebaseMessaging.instance.getToken().then((token) {
        LeooNetConfigJunPro.leooPrintLogJunPro(
            'FirebaseMessaging getToken: $token');
        if (token?.isEmpty == false) {
          EMClient.getInstance.pushManager.updateFCMPushToken(token!);
        }
      });
      FirebaseMessaging.instance.onTokenRefresh.listen((token) {
        LeooNetConfigJunPro.leooPrintLogJunPro(
            'FirebaseMessaging onTokenRefresh: $token');
        if (token.isEmpty == false) {
          EMClient.getInstance.pushManager.updateFCMPushToken(token);
        }
      });
    } catch (e) {
      LeooNetConfigJunPro.leooPrintLogJunPro(e);
    }
  }

  @override
  void onConnected() {
    LeooNetConfigJunPro.leooPrintLogJunPro('EMClient onConnected');
    for (var element in imPluginDelegateList) {
      element.onConnectionStatusChange(true);
    }
    try {
      if (!updateOwnUserInfo) {
        EMClient.getInstance.chatManager.getConversationsFromServer();
        Map map = {
          'userId':
              SpUtil.getInt(LeooAppConfigJunPro.leooUserIdJunPro).toString(),
          'nickName': name,
          'avatarUrl': headImage
        };
        EMUserInfo userInfo = EMUserInfo.fromJson(map);
        EMClient.getInstance.userInfoManager.updateOwnUserInfo(userInfo);
        EMClient.getInstance.pushManager.updatePushNickname(name!);
        updateOwnUserInfo = true;
        initLoneFirebaseLy();
      }
    } catch (e) {
      LeooNetConfigJunPro.leooPrintLogJunPro(e);
    }
  }

  @override
  void onDisconnected(int? errorCode) {
    LeooNetConfigJunPro.leooPrintLogJunPro(
        'EMClient onDisconnected: $errorCode');
    FirebaseAnalytics.instance
        .logEvent(name: '环信链接失败${'$errorCode'.replaceAll('-', '_')}');
    if ((errorCode == 204 || //USER_NOT
            errorCode == 207 || //USER_REMOVED
            errorCode == 206 || //USER_LOGIN_ANOTHER_DEVICE
            errorCode == 305 || //SERVER_SERVICE_RESTRICTED
            errorCode == 216 || //USER_KICKED_BY_CHANGE_PASSWORD
            errorCode == 217) && //USER_KICKED_BY_OTHER_DEVICE
        !LeooAppConfigJunPro.leooSplashOpenJunPro) {
      if (!delete) {
        LeooCommonUtilsJunPro.leooShowCustomToastJunPro(
            S.of(LeooRouterJunPro.leooContextJunPro).chatImOtherLogin);
        LeooNavigatorUtilsJunPro.leooGoSplash2JunPro();
      } else {
        delete = false;
      }
      return;
    }
    for (var element in imPluginDelegateList) {
      element.onConnectionStatusChange(false);
    }
  }

  @override
  void onTokenDidExpire() {
    if (!delete) {
      LeooCommonUtilsJunPro.leooShowCustomToastJunPro(
          S.of(LeooRouterJunPro.leooContextJunPro).chatImOtherLogin);
      LeooNavigatorUtilsJunPro.leooGoSplash2JunPro();
    } else {
      delete = false;
    }
  }

  @override
  void onTokenWillExpire() {}

  @override
  void onMessagesReceived(List<EMMessage> messages) {
    List<EMMessage> chatMessages = messages
        .where((element) => element.chatType != ChatType.ChatRoom)
        .toList();
    if (chatMessages.isNotEmpty) {
      for (var element in imPluginDelegateList) {
        element.onMessageReceived(messages);
      }
    }
  }

  @override
  void onCmdMessagesReceived(List<EMMessage> messages) {
    LeooNetConfigJunPro.leooPrintLogJunPro('onCmdMessagesReceived');
    return super.onCmdMessagesReceived(messages);
  }

  @override
  void onMessagesRead(List<EMMessage> messages) {
    LeooNetConfigJunPro.leooPrintLogJunPro('onMessagesRead');
  }

  @override
  void onMessagesDelivered(List<EMMessage> messages) {
    LeooNetConfigJunPro.leooPrintLogJunPro('onMessagesDelivered');
  }

  @override
  void onMessagesRecalled(List<EMMessage> messages) {
    LeooNetConfigJunPro.leooPrintLogJunPro('onMessagesRecalled');
  }

  @override
  void onConversationsUpdate() {
    LeooNetConfigJunPro.leooPrintLogJunPro('onConversationsUpdate');
    eventBus.fire(LeooMainUnReadEventJunPro());
  }

  @override
  void onConversationRead(String? from, String? to) {
    LeooNetConfigJunPro.leooPrintLogJunPro('onConversationRead');
  }

  void initIMClient() {
    EMClient.getInstance.addConnectionListener(this);
    EMClient.getInstance.chatManager.addChatManagerListener(this);
  }

  String? name;
  String? headImage;

  Future connectIM(
      {BuildContext? context,
      String? userId,
      String? name,
      String? headImage}) async {
    if (userId?.isEmpty == false) {
      updateOwnUserInfo = false;
      this.name = name;
      this.headImage = headImage;
      try {
        if (await EMClient.getInstance.isLoginBefore() == false) {
          await EMClient.getInstance.login(userId!, userId);
        }
      } catch (e) {
        LeooNetConfigJunPro.leooPrintLogJunPro(e);
      }
    } else {
      imPluginDelegateList.clear();
      try {
        EMClient.getInstance.logout(false);
      } catch (e) {
        LeooNetConfigJunPro.leooPrintLogJunPro(e);
      }
    }
  }
}
