import 'dart:async';
import 'dart:io';

import 'package:audioplayers/audioplayers.dart';
import 'package:camera/camera.dart';
import 'package:device_info/device_info.dart';
import 'package:firebase_analytics/firebase_analytics.dart';
import 'package:flutter_secure_storage/flutter_secure_storage.dart';
import 'package:leoo_junpro/generated/l10n.dart';
import 'package:leoo_junpro/leooflagjunpro/leooappjunpro/leooappconfigjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooappjunpro/leoonetconfigjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooutilsjunpro/leoodialogutilsjunpro.dart';
import 'package:package_info/package_info.dart';
import 'package:permission_handler/permission_handler.dart';
import 'package:vibration/vibration.dart';

//媒体工具，负责申请权限，选照片，拍照，录音，播放语音
class LeooMediaUtilsJunPro {
  factory LeooMediaUtilsJunPro() => _getInstance();

  static LeooMediaUtilsJunPro get instance => _getInstance();
  static LeooMediaUtilsJunPro? _instance;

  LeooMediaUtilsJunPro._internal();

  static LeooMediaUtilsJunPro _getInstance() {
    _instance ??= LeooMediaUtilsJunPro._internal();
    return _instance!;
  }

  CameraDescription? description;
  CameraDescription? descriptionBack;

  //请求权限: 相册，相机，麦克风 index 0相机 1相册 2麦克风 3文件 4电话
  Future<bool?> grantedPermissions(context,
      {bool chat = false, int index = -1}) async {
    bool have = true;
    PermissionStatus status;
    if (chat) {
      status = await Permission.camera.status;
      if (status != PermissionStatus.granted) {
        have = false;
      }
      if (checkDenied(status)) {
        return null;
      }
      status = await Permission.microphone.status;
      if (status != PermissionStatus.granted) {
        have = false;
      }
      if (checkDenied(status)) {
        return null;
      }
    } else if (index == 0) {
      status = await Permission.camera.status;
      if (status != PermissionStatus.granted) {
        have = false;
      }
      if (checkDenied(status)) {
        return null;
      }
    } else if (index == 1) {
      status = Platform.isAndroid
          ? await Permission.storage.status
          : await Permission.photos.status;
      if (status != PermissionStatus.granted &&
          status != PermissionStatus.limited) {
        have = false;
      }
      if (checkDenied(status)) {
        return null;
      }
    } else if (index == 2) {
      status = await Permission.microphone.status;
      if (status != PermissionStatus.granted) {
        have = false;
      }
      if (checkDenied(status)) {
        return null;
      }
    } else if (index == 4) {
      status = await Permission.phone.status;
      if (status != PermissionStatus.granted) {
        have = false;
      }
      if (checkDenied(status)) {
        return null;
      }
    } else {
      status = await Permission.storage.status;
      if (status != PermissionStatus.granted &&
          status != PermissionStatus.limited) {
        have = false;
      }
      if (checkDenied(status)) {
        return null;
      }
    }
    return have;
  }

  bool checkDenied(PermissionStatus status) {
    return Platform.isAndroid && status == PermissionStatus.permanentlyDenied ||
        Platform.isIOS && status == PermissionStatus.restricted;
  }

  void showDenied(context, bool chat, int index) {
    LeooDialogUtilsJunPro.instance
        .showCustomDialog(context,
            content: chat
                ? S.of(context).perDialogTitle
                : index == 0
                    ? S.of(context).perDialogCameraTitle
                    : index == 1
                        ? S.of(context).perDialogPhotoTitle
                        : index == 2
                            ? S.of(context).perDialogMicrophoneTitle
                            : S.of(context).perDialogStorageTitle,
            left: S.of(context).appCancel,
            right: S.of(context).perDialogSetting,
            dismiss: false)
        .then((value) {
      if (value == true) {
        openAppSettings();
      }
    });
  }

  //请求权限:0相机，1相册，2麦克风
  Future requestPermissions(context, {bool chat = true, int index = -1}) async {
    Map<Permission, PermissionStatus> rest;
    if (chat) {
      rest = await [
        Permission.camera,
        Permission.microphone,
      ].request();
    } else if (index == 0) {
      rest = await [
        Permission.camera,
      ].request();
    } else if (index == 1) {
      rest = await [
        Platform.isAndroid ? Permission.storage : Permission.photos,
      ].request();
    } else if (index == 2) {
      rest = await [
        Permission.microphone,
      ].request();
    } else if (index == 4) {
      rest = await [
        Permission.phone,
      ].request();
    } else {
      rest = await [
        Permission.storage,
      ].request();
    }
    LeooNetConfigJunPro.leooPrintLogJunPro(rest);
    bool denied = rest.containsValue(PermissionStatus.permanentlyDenied) ||
        rest.containsValue(PermissionStatus.restricted);
    if (denied) {
      showDenied(context, chat, index);
    } else {
      if (rest[Permission.camera] == PermissionStatus.granted) {
        FirebaseAnalytics.instance.logEvent(name: '用户同意摄像头权限');
      } else {
        FirebaseAnalytics.instance.logEvent(name: '用户拒绝摄像头权限');
      }
      if (rest[Permission.microphone] == PermissionStatus.granted) {
        FirebaseAnalytics.instance.logEvent(name: '用户同意麦克风权限');
      } else {
        FirebaseAnalytics.instance.logEvent(name: '用户拒绝麦克风权限');
      }
    }
    bool allOk = true;
    for (var element in rest.values) {
      if (element != PermissionStatus.granted) {
        allOk = false;
      }
    }
    return allOk;
  }

  void requestPermissionsNotification() async {
    try {
      PermissionStatus status = await Permission.notification.status;
      if (status == PermissionStatus.denied) {
        PermissionStatus rest = await Permission.notification.request();
        LeooNetConfigJunPro.leooPrintLogJunPro(rest);
        if (rest == PermissionStatus.granted) {
          FirebaseAnalytics.instance.logEvent(name: '用户同意通知权限');
        } else {
          FirebaseAnalytics.instance.logEvent(name: '用户拒绝通知权限');
        }
      } else if (status == PermissionStatus.granted) {
        FirebaseAnalytics.instance.logEvent(name: '用户同意通知权限');
      }
    } catch (e) {
      LeooNetConfigJunPro.leooPrintLogJunPro(e);
    }
  }

  late IosDeviceInfo iosDeviceInfo;
  late AndroidDeviceInfo androidDeviceInfo;
  late PackageInfo packageInfo;

  Future<bool> getDeviceInfo() async {
    DeviceInfoPlugin deviceInfo = DeviceInfoPlugin();
    if (Platform.isAndroid) {
      androidDeviceInfo = await deviceInfo.androidInfo;
    } else if (Platform.isIOS) {
      iosDeviceInfo = await deviceInfo.iosInfo;
      FlutterSecureStorage storage = const FlutterSecureStorage();
      LeooAppConfigJunPro.leooAppIdJunPro =
          await storage.read(key: LeooNetConfigJunPro.leooIosIdJunPro);
      if (LeooAppConfigJunPro.leooAppIdJunPro?.isEmpty != false) {
        LeooAppConfigJunPro.leooAppIdJunPro = iosDeviceInfo.identifierForVendor;
        storage.write(
            key: LeooNetConfigJunPro.leooIosIdJunPro,
            value: LeooAppConfigJunPro.leooAppIdJunPro);
      }
    }
    packageInfo = await PackageInfo.fromPlatform();
    return true;
  }

  AudioPlayer? audioPlayer;
  AudioCache audioCache = AudioCache();

  //播放语音-主叫
  void startPlayAudio(bool call) async {
    try {
      audioPlayer ??= await audioCache.loop(call
          ? LeooAppConfigJunPro.leooVideoCallJunPro
          : LeooAppConfigJunPro.leooInCallJunPro);
      if (audioPlayer?.state != PlayerState.PLAYING) {
        audioPlayer?.resume();
      }
    } catch (e) {
      LeooNetConfigJunPro.leooPrintLogJunPro(e);
    }
  }

  //停止播放语音
  Future stopPlayAudio([bool audio = true]) async {
    if (Platform.isAndroid) {
      Vibration.cancel();
    } else {
      vTimer?.cancel();
    }
    if (audio && audioPlayer != null) {
      int stop = 0;
      try {
        stop = await audioPlayer!.stop();
      } catch (e) {
        LeooNetConfigJunPro.leooPrintLogJunPro(e);
      }
      if (stop != 1) {
        stopPlayAudio();
      }
    }
  }

  Timer? vTimer;

  void startVibration({bool vibrate = true, repeat = 0}) async {
    if (vibrate && await Vibration.hasVibrator() == true) {
      if (repeat == -1) {
        Vibration.vibrate(pattern: [500, 500], amplitude: 255);
      } else {
        if (Platform.isAndroid) {
          Vibration.vibrate(
              pattern: [500, 500], repeat: repeat, amplitude: 255);
        } else {
          vTimer?.cancel();
          vTimer = Timer.periodic(const Duration(milliseconds: 1000), (timer) {
            Vibration.vibrate(pattern: [500, 500], amplitude: 255);
          });
        }
      }
    }
  }

  //播放消息来提示
  void startPlayMessageTip() {
    try {
      audioCache.play(LeooAppConfigJunPro.leooMessageTipJunPro);
    } catch (e) {
      LeooNetConfigJunPro.leooPrintLogJunPro(e);
    }
  }
}
