import 'dart:convert';
import 'dart:core';
import 'dart:io';

import 'package:agora_rtm/agora_rtm.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:im_flutter_sdk/im_flutter_sdk.dart';
import 'package:leoo_junpro/generated/l10n.dart';
import 'package:leoo_junpro/leooflagjunpro/leooappjunpro/leooappconfigjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooappjunpro/leoonetconfigjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leoodaojunpro/leoomaindaojunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leoodbjunpro/leooproviderjunpro/leoouserinfodbproviderjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooentityjunpro/leoohostinfoentity.dart';
import 'package:leoo_junpro/leooflagjunpro/leooentityjunpro/leoortmentity.dart';
import 'package:leoo_junpro/leooflagjunpro/leooentityjunpro/leoouserentity.dart';
import 'package:leoo_junpro/leooflagjunpro/leoonetjunpro/leooapphostjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leoostringjunpro/leoostylejunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooutilsjunpro/leoocodeutilsjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooutilsjunpro/leoocommonutilsjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooutilsjunpro/leoodialogutilsjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooutilsjunpro/leoomediautilsjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooutilsjunpro/leoonavigatorutilsjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooutilsjunpro/leoorechargeutilsjunpro.dart';
import 'package:leoo_junpro/leooimpjunpro/leoocalljunpro/leoochatjunpro.dart';
import 'package:leoo_junpro/leooimpjunpro/leoocalljunpro/leooitemjunpro/leoocallentity.dart';
import 'package:leoo_junpro/leooimpjunpro/leoocalljunpro/leoovideocallhostjunpro.dart';
import 'package:leoo_junpro/leooimpjunpro/leoocalljunpro/leoovideocalljunpro.dart';
import 'package:leoo_junpro/leooimpjunpro/leoocalljunpro/leoovideocallmodejunpro.dart';
import 'package:leoo_junpro/leooimpjunpro/leoocalljunpro/leoovideoendjunpro.dart';
import 'package:leoo_junpro/leooimpjunpro/leooinfojunpro/leoovideoplayjunpro.dart';
import 'package:leoo_junpro/leooimpjunpro/leoomainjunpro/leoomainjunpro.dart';
import 'package:leoo_junpro/leooimpjunpro/leoomatjunpro/leoomatinfojunpro.dart';
import 'package:overlay_support/overlay_support.dart';
import 'package:wakelock/wakelock.dart';

//定义一个抽象类-聊天监听，视频通话逻辑
abstract class OnRtmPluginDelegate {
  void onRtmMessageReceived(LeooRtmEntity rtmEntity);

  void onRemoteInvitationCanceled(AgoraRtmRemoteInvitation invite);
}

///视频前置方法
class LeooVideoUtilsJunPro {
  factory LeooVideoUtilsJunPro() => _getInstance();

  static LeooVideoUtilsJunPro get instance => _getInstance();
  static LeooVideoUtilsJunPro? _instance;

  LeooVideoUtilsJunPro._internal();

  static LeooVideoUtilsJunPro _getInstance() {
    _instance ??= LeooVideoUtilsJunPro._internal();
    return _instance!;
  }

  bool isReception = true;

  AgoraRtmClient? rtmClient;
  AgoraRtmChannel? rtmChannel;
  String? rtmLoginUserId;
  bool rtmLogin = false;
  bool rtmLoginOut = false; //是否需要重新登录
  String? channelId;
  LeooUserInfoDbProviderJunPro? ubProvider;
  List<OnRtmPluginDelegate> rtmPluginDelegateList = [];

  LeooUserInfoDbProviderJunPro get userDbProvider {
    ubProvider ??= LeooUserInfoDbProviderJunPro();
    return ubProvider!;
  }

  void createRtmClient(context, {String? userId}) async {
    try {
      if (rtmClient == null) {
        rtmClient = await AgoraRtmClient.createInstance(
            LeooNetConfigJunPro.leooAgoraAppKeyJunPro);
        rtmClient!.onMessageReceived =
            (AgoraRtmMessage message, String? peerId) async {
          LeooNetConfigJunPro.leooPrintLogJunPro(
              'RtmClient peerId: $peerId, msg: ${message.text}');
          if (message.text.startsWith("{")) {
            var rtmEntity = LeooRtmEntity.fromJson(jsonDecode(message.text));
            for (var element in rtmPluginDelegateList) {
              element.onRtmMessageReceived(rtmEntity);
            }
          }
        };
        rtmClient!.onConnectionStateChanged = (int? state, int? reason) {
          LeooNetConfigJunPro.leooPrintLogJunPro(
              'onConnectionStateChanged: $state, reason: $reason');
          rtmLogin = reason == 2 && state == 3;
          rtmLoginOut = reason == 6 || state == 5 || state == 1;
        };
        rtmClient!.onTokenExpired = () async {
          String token = await getRtmToken();
          if (token.isEmpty == false) {
            rtmClient!.renewToken(token);
          }
        };
      }
      onReceivedByPeer(context);
      if (userId != null) {
        rtmLoginUserId = userId;
        loginRtmClient(context, userId: userId);
      }
    } catch (e) {
      LeooNetConfigJunPro.leooPrintLogJunPro(e);
    }
  }

  void receivedByPeerShow(context, EMMessage message) async {
    Map<String, EMUserInfo?> userInfoMap = await EMClient
        .getInstance.userInfoManager
        .fetchUserInfoById([message.from!]);
    String? text =
        LeooCodeUtilsJunPro.leooGetDigestJunPro(message.body, context);
    int userId = int.parse(message.from ?? '0');
    String nickname = userInfoMap[message.from!]?.nickName?.isEmpty == false
        ? userInfoMap[message.from!]?.nickName ?? ''
        : LeooNetConfigJunPro.leooAppNameJunPro;
    String? header = userInfoMap[message.from!]?.avatarUrl;
    if (!LeooVideoUtilsJunPro.instance.isReception) {
      var time = DateTime.fromMillisecondsSinceEpoch(
          DateTime.now().millisecondsSinceEpoch + 500);
      LeooRechargeUtilsJunPro.instance.flutterLocalNotificationsPlugin.show(
          time.millisecond,
          nickname,
          text,
          LeooRechargeUtilsJunPro.instance.platformChannelSpecifics);
    } else if (!LeooChatJunPro.leooOpenJunPro &&
        !LeooVideoCallJunPro.leooOpenJunPro) {
      LeooMediaUtilsJunPro.instance.startPlayMessageTip();
      var time = DateTime.now().millisecondsSinceEpoch;
      showOverlayNotification((BuildContext context) {
        return Container(
          margin: EdgeInsets.only(
              left: 5.w, top: 5.w + ScreenUtil().statusBarHeight, right: 5.w),
          decoration: ShapeDecoration(
            color: LeooColorsJunPro.leooPopBgJunPro,
            shape: RoundedRectangleBorder(
              borderRadius: BorderRadius.circular(20),
            ),
          ),
          child: TextButton(
            style: ButtonStyle(
              padding: MaterialStateProperty.all(
                EdgeInsets.all(15.w),
              ),
            ),
            onPressed: LeooNavigatorUtilsJunPro.leooDebounceJunPro(() {
              OverlaySupportEntry.of(context)?.dismiss();
              LeooUserEntity infoEntity = LeooUserEntity(
                id: userId,
                nickname: nickname,
                header: header,
              );

              LeooNavigatorUtilsJunPro.leooNavigatorPushJunPro(
                  context,
                  LeooChatJunPro(
                      userId, infoEntity, '${userId}notification$time'),
                  'LeooChatJunPro',
                  add: true);
            }),
            child: Row(
              children: <Widget>[
                Hero(
                  tag: '${userId}notification$time',
                  transitionOnUserGestures: true,
                  child: ClipOval(
                    child: LeooCommonUtilsJunPro.leooGetLoadWidgetJunPro(
                      header,
                      width: 50.w,
                      height: 50.w,
                      load: LeooIconsJunPro.leooLoadHeadJunPro,
                    ),
                  ),
                ),
                SizedBox(width: 10.w),
                Expanded(
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                    children: <Widget>[
                      Text(nickname,
                          style: LeooTextJunPro.leooGetTextStyleJunPro(
                              16, FontWeight.bold),
                          maxLines: 1,
                          overflow: TextOverflow.ellipsis),
                      Text(text ?? '',
                          style: LeooTextJunPro.leooGetTextStyleJunPro(12),
                          maxLines: 1,
                          overflow: TextOverflow.ellipsis),
                    ],
                  ),
                ),
                SizedBox(width: 5.w),
                LeooCommonUtilsJunPro.leooGetLoadWidgetJunPro(
                  LeooIconsJunPro.leooItemAddJunPro,
                  width: 20.w,
                  height: 20.w,
                ),
              ],
            ),
          ),
        );
      });
    }
  }

  void onReceivedByPeer(context) {
    rtmClient?.onRemoteInvitationReceivedByPeer =
        (AgoraRtmRemoteInvitation invite) {
      try {
        if (!LeooVideoCallJunPro.leooOpenJunPro &&
            !LeooRechargeUtilsJunPro.instance.showLoading) {
          LeooCallEntity message =
              LeooCallEntity.fromJson(jsonDecode(invite.content!));
          LeooUserEntity infoEntity = LeooUserEntity(
              id: int.parse(invite.callerId),
              nickname: message.userName,
              header: message.userImage);
          requestVideoPermission(context, int.parse(invite.callerId),
              call: false,
              infoEntity: infoEntity,
              callId: int.parse(message.callId ?? '0'),
              channelId: invite.channelId,
              invite: invite);
        } else {
          rtmClient?.refuseRemoteInvitation({'callerId': invite.callerId});
        }
      } catch (e) {
        LeooNetConfigJunPro.leooPrintLogJunPro(e);
      }
    };
    rtmClient?.onRemoteInvitationCanceled = (AgoraRtmRemoteInvitation invite) {
      try {
        for (var element in rtmPluginDelegateList) {
          element.onRemoteInvitationCanceled(invite);
        }
      } catch (e) {
        LeooNetConfigJunPro.leooPrintLogJunPro(e);
      }
    };
    rtmClient?.onLocalInvitationReceivedByPeer = null;
    rtmClient?.onLocalInvitationAccepted = null;
    rtmClient?.onLocalInvitationRefused = null;
  }

  Future<String> getRtmToken() async {
    return LeooMainDaoJunPro.leooRtmTokenJunPro().then((res) async {
      if (res.result) {
        return res.data.token;
      }
      return '';
    });
  }

  void loginRtmClient(context, {String? userId}) async {
    try {
      String token = await getRtmToken();
      if (token.isEmpty != false ||
          userId?.isEmpty != false && rtmLoginUserId?.isEmpty != false) {
        LeooCommonUtilsJunPro.leooShowCustomToastJunPro(
            S.of(context).chatImNoLogin);
        LeooNavigatorUtilsJunPro.leooGoSplashJunPro(context);
        return;
      }
      await rtmClient?.login(token, userId ?? rtmLoginUserId!);
      rtmLogin = true;
    } catch (e) {
      LeooNetConfigJunPro.leooPrintLogJunPro(e);
    }
  }

  Future logoutRtmClient() async {
    try {
      rtmPluginDelegateList.clear();
      await rtmClient?.logout();
      rtmLogin = false;
    } catch (e) {
      LeooNetConfigJunPro.leooPrintLogJunPro(e);
    }
  }

  Future<bool> createRtmChannel(context, {String? channelId}) async {
    try {
      if (!rtmLogin) {
        LeooNetConfigJunPro.leooPrintLogJunPro(
            'rtmLogin: $rtmLogin, rtmLoginOut: $rtmLoginOut');
        LeooCommonUtilsJunPro.leooShowCustomToastJunPro(S.of(context).chatImNo);
        if (rtmLoginOut) {
          rtmLoginOut = false;
          rtmLogin = false;
          rtmClient = null;
          createRtmClient(context);
        }
        return false;
      }
      rtmChannel = await rtmClient?.createChannel(channelId!);
      rtmChannel!.join();
      this.channelId = channelId;
      return true;
    } catch (e) {
      LeooNetConfigJunPro.leooPrintLogJunPro(e);
      return false;
    }
  }

  Future logoutRtmChannel() async {
    try {
      if (channelId?.isEmpty == false) {
        await rtmChannel?.leave();
        channelId = null;
      }
    } catch (e) {
      LeooNetConfigJunPro.leooPrintLogJunPro(e);
    }
  }

  Future<bool?> openApp() async {
    Wakelock.enable();
    const MethodChannel _channel = MethodChannel('flutter_app');
    bool? result = false;
    try {
      result = await _channel.invokeMethod('openApp');
    } on PlatformException catch (e) {
      LeooNetConfigJunPro.leooPrintLogJunPro(e.toString());
    }
    return result;
  }

  void requestVideoPermission(context, int? userId,
      {LeooUserEntity? infoEntity,
      LeooHostInfoEntity? infoHostEntity,
      int? matchingId,
      int? callId,
      String? channelId,
      bool call = true,
      AgoraRtmRemoteInvitation? invite}) async {
    if (await LeooMediaUtilsJunPro.instance
            .grantedPermissions(context, chat: true) ==
        true) {
      if (!rtmLogin) {
        LeooNetConfigJunPro.leooPrintLogJunPro(
            'rtmLogin: $rtmLogin, rtmLoginOut: $rtmLoginOut');
        LeooCommonUtilsJunPro.leooShowCustomToastJunPro(S.of(context).chatImNo);
        if (rtmLoginOut) {
          rtmLoginOut = false;
          rtmLogin = false;
          rtmClient = null;
          createRtmClient(context);
        }
        return;
      }
      if (isReception) {
        startVideoPage(context, userId, call, infoEntity, infoHostEntity,
            matchingId, callId, channelId, invite);
      } else {
        var time = DateTime.fromMillisecondsSinceEpoch(
            DateTime.now().millisecondsSinceEpoch + 500);
        LeooRechargeUtilsJunPro.instance.flutterLocalNotificationsPlugin.show(
            time.millisecond,
            LeooNetConfigJunPro.leooAppNameJunPro,
            S.of(context).notificationsCall,
            LeooRechargeUtilsJunPro.instance.platformChannelSpecifics);
        if (Platform.isAndroid) {
          await openApp();
        }
        Future.delayed(const Duration(milliseconds: 500), () {
          startVideoPage(context, userId, call, infoEntity, infoHostEntity,
              matchingId, callId, channelId, invite);
        });
      }
    } else {
      LeooDialogUtilsJunPro.instance.showPermissionDialog(context, (value) {
        if (value) {
          requestVideoPermission(context, userId,
              infoEntity: infoEntity,
              infoHostEntity: infoHostEntity,
              call: call,
              matchingId: matchingId,
              callId: callId,
              channelId: channelId,
              invite: invite);
        }
      });
    }
  }

  void startVideoPage(
      context,
      int? userId,
      bool call,
      LeooUserEntity? infoEntity,
      LeooHostInfoEntity? infoHostEntity,
      int? matchingId,
      int? callId,
      String? channelId,
      AgoraRtmRemoteInvitation? invite) {
    if (LeooVideoCallJunPro.leooOpenJunPro) {
      if (invite != null) {
        rtmClient?.refuseRemoteInvitation({'callerId': invite.callerId});
      }
      return;
    }
    if (LeooVideoCallJunPro.leooOpenCloseJunPro) {
      LeooVideoCallJunPro.leooOpenCloseJunPro = false;
      return;
    }
    if (LeooMatInfoJunPro.leooOpenJunPro) {
      Navigator.pop(context, 0);
    }
    if (LeooVideoEndJunPro.leooOpenJunPro) {
      Navigator.pop(context);
    }
    if (LeooVideoPlayJunPro.leooOpenJunPro) {
      LeooNavigatorUtilsJunPro.leooPopUntilJunPro(
          context,
          LeooAppConfigJunPro.leooMainNameJunPro ??
              LeooMainJunPro.leooNameJunPro);
    }
    LeooVideoCallJunPro.leooOpenJunPro = true;
    LeooMainDaoJunPro.leooOnlineHeartbeatJunPro(1);
    (!LeooAppHostJunPro.leooModeJunPro
            ? LeooNavigatorUtilsJunPro.leooNavigatorPushJunPro(
                context,
                LeooVideoCallModeJunPro(userId, call,
                    infoEntity: infoEntity, channelId: channelId),
                'LeooVideoCallModeJunPro',
                fullscreenDialog: true,
                match: true,
                center: Offset(.5.sw, .5.sh))
            : infoHostEntity != null
                ? LeooNavigatorUtilsJunPro.leooNavigatorPushJunPro(
                    context,
                    LeooVideoCallHostJunPro(userId, infoHostEntity),
                    'LeooVideoCallHostJunPro',
                    fullscreenDialog: true,
                    match: true,
                    center: Offset(.5.sw, .5.sh))
                : LeooNavigatorUtilsJunPro.leooNavigatorPushJunPro(
                    context,
                    LeooVideoCallJunPro(userId, call,
                        infoEntity: infoEntity,
                        matchingId: matchingId,
                        callId: callId,
                        channelId: channelId),
                    'LeooVideoCallJunPro',
                    fullscreenDialog: true,
                    match: true,
                    center: Offset(.5.sw, .5.sh)))
        .then((value) {
      LeooVideoCallJunPro.leooOpenJunPro = false;
      LeooMainDaoJunPro.leooOnlineHeartbeatJunPro(2);
      LeooMediaUtilsJunPro.instance.stopPlayAudio();
    });
  }

  void sendRtmMessage(int? userId, String? text) {
    try {
      AgoraRtmMessage message = AgoraRtmMessage.fromText(text ?? '');
      LeooVideoUtilsJunPro.instance.rtmClient!
          .sendMessageToPeer('$userId', message, true, true);
    } catch (e) {
      LeooNetConfigJunPro.leooPrintLogJunPro(e);
    }
  }
}
