import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:leoo_junpro/leooflagjunpro/leoostringjunpro/leoostylejunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooutilsjunpro/leoocodeutilsjunpro.dart';

///button
class LeooButtonWidgetJunPro extends StatefulWidget {
  final String? iconText;
  final dynamic iconData;
  final dynamic leftIconData;
  final TextStyle textStyle;
  final double iconPadding;
  final double leftIconPadding;
  final VoidCallback? onPressed;
  final MainAxisAlignment mainAxisAlignment;
  final MainAxisSize mainAxisSize;
  final CrossAxisAlignment crossAxisAlignment;
  final double textWidth;
  final bool textExpanded;
  final TextAlign textAlign;
  final EdgeInsetsGeometry? padding;
  final ShapeBorder? shape;
  final Color? color;
  final Color? colorRight;
  final Color? colorAlpha;
  final double? height;
  final bool row;
  final bool right;
  final bool disabledBg;
  final int maxLines;

  const LeooButtonWidgetJunPro(this.iconText, this.textStyle,
      {Key? key,
      this.iconData,
      this.leftIconData,
      this.iconPadding = 0,
      this.leftIconPadding = 0,
      this.onPressed,
      this.textWidth = -1,
      this.mainAxisAlignment = MainAxisAlignment.center,
      this.mainAxisSize = MainAxisSize.max,
      this.crossAxisAlignment = CrossAxisAlignment.center,
      this.padding,
      this.shape,
      this.color,
      this.colorRight,
      this.colorAlpha,
      this.height,
      this.right = false,
      this.row = true,
      this.disabledBg = false,
      this.textAlign = TextAlign.start,
      this.textExpanded = false,
      this.maxLines = 1})
      : super(key: key);

  @override
  LeooButtonWidgetState createState() => LeooButtonWidgetState();
}

class LeooButtonWidgetState extends State<LeooButtonWidgetJunPro> {
  String? iconText;

  @override
  void initState() {
    super.initState();
    iconText = widget.iconText;
  }

  void onSetState(String text) {
    iconText = text;
    LeooCodeUtilsJunPro.onSetState(this);
  }

  @override
  void didUpdateWidget(LeooButtonWidgetJunPro oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (iconText != widget.iconText) {
      iconText = widget.iconText;
    }
  }

  @override
  Widget build(BuildContext context) {
    Widget showText = (widget.textWidth == -1)
        ? Text(iconText ?? '',
            style: widget.textStyle
                .merge(const TextStyle(textBaseline: TextBaseline.alphabetic)),
            textAlign: widget.textAlign,
            overflow: TextOverflow.ellipsis,
            maxLines: widget.maxLines)
        : SizedBox(
            width: widget.textWidth,
            child: Text(iconText ?? '',
                style: widget.textStyle.merge(
                    const TextStyle(textBaseline: TextBaseline.alphabetic)),
                textAlign: widget.textAlign,
                overflow: TextOverflow.ellipsis,
                maxLines: widget.maxLines));
    Color btnColor = widget.color ?? Colors.transparent;
    Color btnColorAlpha = widget.colorAlpha ??
        (widget.disabledBg ? btnColor : btnColor.withOpacity(.5));
    Color btnSplashColor =
        widget.colorAlpha ?? LeooColorsJunPro.leooBtnBgJunPro.withOpacity(.2);
    if (widget.iconData == null) {
      return InkWell(
        highlightColor: btnSplashColor,
        child: Container(
          height: widget.height,
          decoration: ShapeDecoration(
              gradient: LinearGradient(
                colors: widget.onPressed == null
                    ? [btnColorAlpha, btnColorAlpha]
                    : [
                        btnColor,
                        widget.colorRight ?? btnColor,
                      ],
              ),
              shape: widget.shape ?? RoundedRectangleBorder()),
          padding: widget.height != null
              ? null
              : widget.padding ?? EdgeInsets.fromLTRB(20.w, 14.w, 20.w, 14.w),
          child: Row(
            textBaseline: TextBaseline.alphabetic,
            mainAxisAlignment: widget.mainAxisAlignment,
            mainAxisSize: widget.mainAxisSize,
            crossAxisAlignment: widget.crossAxisAlignment,
            children: <Widget>[
              widget.textExpanded ? Expanded(child: showText) : showText
            ],
          ),
        ),
        onTap: widget.onPressed,
      );
    } else {
      return InkWell(
        highlightColor: btnSplashColor,
        child: Container(
          height: widget.height,
          decoration: ShapeDecoration(
              gradient: LinearGradient(
                colors: widget.onPressed == null
                    ? [btnColorAlpha, btnColorAlpha]
                    : [
                        btnColor,
                        widget.colorRight ?? btnColor,
                      ],
              ),
              shape: widget.shape ?? RoundedRectangleBorder()),
          padding: widget.height != null
              ? null
              : widget.padding ?? EdgeInsets.fromLTRB(20.w, 14.w, 20.w, 14.w),
          child: widget.row
              ? Row(
                  textBaseline: TextBaseline.alphabetic,
                  mainAxisAlignment: widget.mainAxisAlignment,
                  mainAxisSize: widget.mainAxisSize,
                  crossAxisAlignment: widget.crossAxisAlignment,
                  children: (widget.right
                      ? <Widget>[
                          widget.leftIconData ?? Container(),
                          Padding(
                              padding: EdgeInsets.all(widget.leftIconPadding)),
                          widget.textExpanded
                              ? Expanded(child: showText)
                              : showText,
                          Padding(padding: EdgeInsets.all(widget.iconPadding)),
                          widget.iconData
                        ]
                      : <Widget>[
                          widget.iconData,
                          Padding(padding: EdgeInsets.all(widget.iconPadding)),
                          widget.textExpanded
                              ? Expanded(child: showText)
                              : showText
                        ]),
                )
              : Column(
                  textBaseline: TextBaseline.alphabetic,
                  mainAxisAlignment: widget.mainAxisAlignment,
                  mainAxisSize: widget.mainAxisSize,
                  crossAxisAlignment: widget.crossAxisAlignment,
                  children: (widget.right
                      ? <Widget>[
                          widget.leftIconData ?? Container(),
                          Padding(
                              padding: EdgeInsets.all(widget.leftIconPadding)),
                          widget.textExpanded
                              ? Expanded(child: showText)
                              : showText,
                          Padding(padding: EdgeInsets.all(widget.iconPadding)),
                          widget.iconData
                        ]
                      : <Widget>[
                          widget.iconData,
                          Padding(padding: EdgeInsets.all(widget.iconPadding)),
                          widget.textExpanded
                              ? Expanded(child: showText)
                              : showText
                        ]),
                ),
        ),
        onTap: widget.onPressed,
      );
    }
  }
}
