import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:leoo_junpro/leooflagjunpro/leoostringjunpro/leoostylejunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooutilsjunpro/leoocodeutilsjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooutilsjunpro/leoocommonutilsjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooutilsjunpro/leoonavigatorutilsjunpro.dart';

///AttackInputWidgetDel
typedef AttackInputWidgetDelCallBack = void Function(String content);

enum ITextInputType {
  text,
  multiline,
  number,
  phone,
  datetime,
  emailAddress,
  url,
  password,
  amount,
}

class LeooEditTextWidgetJunPro extends StatefulWidget {
  final ITextInputType keyboardType;
  final int? minLines;
  final int maxLines;
  final int? inputMaxLength;
  final String inputText;
  final String? hintText;
  final TextStyle? hintStyle;
  final AttackInputWidgetDelCallBack? fieldCallBack;
  final Icon? deleteIcon;
  final InputBorder? inputBorder;
  final InputBorder? focusedBorder;
  final Color? fillColor;
  final Widget? prefixIcon;
  final Widget? suffixIcon;
  final String? suffixStrShow;
  final String? suffixStrHide;
  final TextStyle? textStyle;
  final TextAlign textAlign;
  final FormFieldValidator<String>? validator;
  final bool enabled;
  final ValueChanged<String>? onSubmitted;
  final FocusNode? focusNode;
  final EdgeInsets? contentPadding;
  final bool letter;

  const LeooEditTextWidgetJunPro({
    Key? key,
    ITextInputType keyboardType = ITextInputType.text,
    this.minLines,
    this.maxLines = 1,
    this.hintText,
    this.hintStyle,
    this.fieldCallBack,
    this.deleteIcon,
    this.inputBorder,
    this.textStyle,
    this.suffixIcon,
    this.validator,
    this.fillColor,
    this.inputText = '',
    this.textAlign = TextAlign.start,
    this.enabled = true,
    this.inputMaxLength,
    this.onSubmitted,
    this.focusNode,
    this.contentPadding,
    this.letter = false,
    this.focusedBorder,
    this.prefixIcon,
    this.suffixStrShow,
    this.suffixStrHide,
  })  : assert(maxLines > 0),
        keyboardType = maxLines == 1 ? keyboardType : ITextInputType.multiline,
        super(key: key);

  @override
  State<StatefulWidget> createState() => _LeooEditTextWidgetState();
}

class _LeooEditTextWidgetState extends State<LeooEditTextWidgetJunPro> {
  String _inputText = '';
  bool _isNumber = false;
  bool _isPasswordShow = false;
  int _isCard = 0;
  final TextEditingController _textEditingController = TextEditingController();
  late ITextInputType keyboardType;

  ///输入类型
  // ignore: missing_return
  TextInputType _getTextInputType() {
    switch (keyboardType) {
      case ITextInputType.text:
        return TextInputType.text;
      case ITextInputType.multiline:
        return TextInputType.multiline;
      case ITextInputType.number:
        _isNumber = true;
        return TextInputType.number;
      case ITextInputType.phone:
        _isNumber = true;
        return TextInputType.phone;
      case ITextInputType.datetime:
        return TextInputType.datetime;
      case ITextInputType.emailAddress:
        return TextInputType.emailAddress;
      case ITextInputType.url:
        return TextInputType.url;
      case ITextInputType.password:
        _isPasswordShow = true;
        return TextInputType.text;
      case ITextInputType.amount:
        _isCard = 1;
        return TextInputType.number;
    }
  }

  ///输入范围
  List<TextInputFormatter> _getTextInputFormatter() {
    List<TextInputFormatter> list = [
      LengthLimitingTextInputFormatter(widget.inputMaxLength)
    ];
    if (_isCard == 1) {
      list.add(FilteringTextInputFormatter.allow(RegExp("[0-9.]")));
    } else if (_isNumber) {
      list.add(FilteringTextInputFormatter.digitsOnly);
    } else if (widget.letter) {
      list.add(FilteringTextInputFormatter.allow(RegExp('^[a-z0-9A-Z]+')));
    }
    return list;
  }

  @override
  void initState() {
    super.initState();
    _inputText = widget.inputText;
    keyboardType = widget.keyboardType;
    _textEditingController.text = _inputText;
  }

  ///外部文字更新之后更新内部文字
  @override
  void didUpdateWidget(LeooEditTextWidgetJunPro oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_inputText != widget.inputText) {
      _inputText = widget.inputText;
      _textEditingController.text = _inputText;
      _textEditingController.selection = TextSelection.fromPosition(
          TextPosition(
              affinity: TextAffinity.downstream, offset: _inputText.length));
    }
  }

  @override
  Widget build(BuildContext context) {
    return TextField(
        cursorColor: LeooColorsJunPro.leooAppJunPro,
        controller: _textEditingController,
        decoration: InputDecoration(
            hintStyle: widget.hintStyle,
            hintText: widget.hintText,
            contentPadding: widget.contentPadding ?? EdgeInsets.all(15.w),
            disabledBorder: widget.inputBorder ??
                const UnderlineInputBorder(borderSide: BorderSide.none),
            enabledBorder: widget.inputBorder ??
                const UnderlineInputBorder(borderSide: BorderSide.none),
            focusedBorder: widget.focusedBorder ??
                widget.inputBorder ??
                const UnderlineInputBorder(borderSide: BorderSide.none),
            fillColor: widget.fillColor ?? Colors.transparent,
            filled: true,
            prefixIcon: widget.prefixIcon,
            suffixIcon: _inputText.isEmpty
                ? null
                : widget.suffixIcon != null
                    ? InkWell(
                        onTap: LeooNavigatorUtilsJunPro.leooDebounceJunPro(() {
                          _isPasswordShow = !_isPasswordShow;
                          if (_isPasswordShow) {
                            keyboardType = ITextInputType.password;
                          } else {
                            keyboardType = ITextInputType.text;
                          }
                          LeooCodeUtilsJunPro.onSetState(this);
                        }),
                        child: widget.suffixIcon,
                      )
                    : widget.suffixStrShow != null
                        ? InkWell(
                            onTap:
                                LeooNavigatorUtilsJunPro.leooDebounceJunPro(() {
                              _isPasswordShow = !_isPasswordShow;
                              if (_isPasswordShow) {
                                keyboardType = ITextInputType.password;
                              } else {
                                keyboardType = ITextInputType.text;
                              }
                              LeooCodeUtilsJunPro.onSetState(this);
                            }),
                            child: Padding(
                              padding: EdgeInsets.all(12.w),
                              child:
                                  LeooCommonUtilsJunPro.leooGetLoadWidgetJunPro(
                                _isPasswordShow
                                    ? widget.suffixStrHide!
                                    : widget.suffixStrShow!,
                                width: 12.w,
                                height: 12.w,
                                fit: BoxFit.contain,
                              ),
                            ),
                          )
                        : null),
        onChanged: (str) {
          _inputText = str;
          widget.fieldCallBack!(_inputText);
          LeooCodeUtilsJunPro.onSetState(this);
        },
        focusNode: widget.focusNode ?? FocusNode(),
        textAlign: widget.textAlign,
        keyboardType: _getTextInputType(),
        minLines: widget.minLines,
        maxLines: widget.maxLines,
        inputFormatters: _getTextInputFormatter(),
        style: widget.textStyle,
        obscureText: _isPasswordShow,
        enabled: widget.enabled,
        onSubmitted: widget.onSubmitted);
  }
}
