import 'dart:math';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:leoo_junpro/leooflagjunpro/leooappjunpro/leoonetconfigjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooutilsjunpro/leoocodeutilsjunpro.dart';

///翻转卡片的控件
class LeooFlipWidgetJunPro extends StatefulWidget {
  final Widget frontChild; //正面
  final Widget backChild; //反面
  final double? indexX;
  final double? indexY;
  final int indexLeft;

  const LeooFlipWidgetJunPro(this.frontChild, this.backChild,
      {Key? key, this.indexX, this.indexY, this.indexLeft = 0})
      : super(key: key);

  @override
  _LeooFlipWidgetState createState() => _LeooFlipWidgetState();
}

class _LeooFlipWidgetState extends State<LeooFlipWidgetJunPro>
    with TickerProviderStateMixin {
  AnimationController? controller;
  late Animation positionAnimation;
  late Animation scaleAnimation;

  ///翻转动画
  AnimationController? animationController;
  late Animation frontAnimation;
  late Animation backAnimation;

  bool isFront = true;
  bool hasHalf = false;

  @override
  void initState() {
    controller = AnimationController(
        vsync: this, duration: const Duration(milliseconds: 1000))
      ..addStatusListener((status) {
        if (status == AnimationStatus.completed) {
          LeooCodeUtilsJunPro.onSetState(this);
          Future.delayed(Duration(milliseconds: 500 * (8 - widget.indexLeft)),
              () {
            try {
              animationController?.forward();
            } catch (e) {
              LeooNetConfigJunPro.leooPrintLogJunPro(e);
            }
          });
        }
      });
    positionAnimation = RelativeRectTween(
            begin: RelativeRect.fromSize(
                Rect.fromLTWH(.5.sw - 100.w, .3.sh - 140.w, 100.w, 140.w),
                Size.zero),
            end: RelativeRect.fromSize(
                Rect.fromLTWH(
                    .5.sw - 100.w + (widget.indexX! + .5) * 112.w,
                    .3.sh - 140.w + (widget.indexY! + .5) * 150.w,
                    100.w,
                    140.w),
                Size.zero))
        .animate(CurvedAnimation(parent: controller!, curve: Curves.linear));
    scaleAnimation = Tween(begin: .7, end: 1.0)
        .animate(CurvedAnimation(parent: controller!, curve: Curves.linear));
    animationController = AnimationController(
        vsync: this, duration: const Duration(milliseconds: 600))
      ..addListener(() {
        if (animationController!.value > .5 && hasHalf == false) {
          isFront = false;
          hasHalf = true;
          LeooCodeUtilsJunPro.onSetState(this);
        }
      });
    frontAnimation = Tween(begin: .0, end: .5).animate(
      CurvedAnimation(
        parent: animationController!,
        curve: const Interval(.0, .5, curve: Curves.easeIn),
      ),
    );
    backAnimation = Tween(begin: 1.5, end: 2.0).animate(
      CurvedAnimation(
        parent: animationController!,
        curve: const Interval(.5, 1.0, curve: Curves.easeOut),
      ),
    );
    Future.delayed(Duration(milliseconds: 400 * widget.indexLeft), () {
      try {
        controller?.forward();
      } catch (e) {
        LeooNetConfigJunPro.leooPrintLogJunPro(e);
      }
    });
    super.initState();
  }

  @override
  void dispose() {
    controller?.dispose();
    animationController?.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    if (controller!.status == AnimationStatus.completed) {
      return Positioned(
        left: .5.sw - 100.w + (widget.indexX! + .5) * 112.w,
        top: .3.sh - 140.w + (widget.indexY! + .5) * 150.w,
        child: AnimatedBuilder(
          animation: hasHalf ? backAnimation : frontAnimation,
          builder: (BuildContext context, Widget? child) {
            var value = (hasHalf ? backAnimation : frontAnimation).value;
            return Transform(
                transform: Matrix4.identity()
                  ..setEntry(3, 2, .002)
                  ..rotateY(pi * value),
                alignment: Alignment.center,
                child: child);
          },
          child: IndexedStack(
            index: isFront ? 0 : 1,
            children: <Widget>[widget.frontChild, widget.backChild],
          ),
        ),
      );
    } else {
      return PositionedTransition(
        rect: positionAnimation as Animation<RelativeRect>,
        child: ScaleTransition(
          scale: scaleAnimation as Animation<double>,
          child: AnimatedBuilder(
            animation: hasHalf ? backAnimation : frontAnimation,
            builder: (BuildContext context, Widget? child) {
              var value = (hasHalf ? backAnimation : frontAnimation).value;
              return Transform(
                  transform: Matrix4.identity()
                    ..setEntry(3, 2, .002)
                    ..rotateY(pi * value),
                  alignment: Alignment.center,
                  child: child);
            },
            child: IndexedStack(
              index: isFront ? 0 : 1,
              children: <Widget>[widget.frontChild, widget.backChild],
            ),
          ),
        ),
      );
    }
  }
}
