import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:leoo_junpro/leooflagjunpro/leooutilsjunpro/leoocodeutilsjunpro.dart';

///拖动控件
class LeooMoveStackWidgetJunPro extends StatefulWidget {
  final Widget child;
  final double xPosition;
  final double yPosition;
  final double? maxY;
  final double? width;
  final bool update;
  final bool chat;

  final bool? right; //吸附右边，false左边 null不吸附
  final bool half; //展示半边

  const LeooMoveStackWidgetJunPro(this.child, this.xPosition, this.yPosition,
      {Key? key,
      this.update = true,
      this.maxY,
      this.chat = false,
      this.right,
      this.half = false,
      this.width})
      : super(key: key);

  @override
  State<StatefulWidget> createState() => LeooMoveStackWidgetState();
}

class LeooMoveStackWidgetState extends State<LeooMoveStackWidgetJunPro> {
  double xPosition = 0;
  double yPosition = 0;
  double x = 0;
  double y = 0;
  double? maxY = 0;
  double keyboardHeight = 0;
  Timer? timer;
  bool showHalf = false;

  @override
  void initState() {
    x = widget.xPosition;
    y = widget.yPosition;
    maxY = widget.maxY;
    xPosition = widget.xPosition;
    yPosition = widget.yPosition;
    super.initState();
    if (widget.half) {
      onHalf(false);
    }
  }

  @override
  void dispose() {
    timer?.cancel();
    super.dispose();
  }

  void onSetState(double keyboardHeight) {
    this.keyboardHeight = keyboardHeight;
    if (widget.update) {
      y = widget.yPosition;
      if (y == maxY) {
        if (yPosition + keyboardHeight > y) {
          yPosition = maxY! - keyboardHeight;
        }
      }
    }
    LeooCodeUtilsJunPro.onSetState(this);
  }

  void onHalf(bool left) {
    showHalf = false;
    timer?.cancel();
    timer = Timer(const Duration(seconds: 2), () {
      if (left) {
        xPosition = -(widget.width ?? 0) / 2;
      } else {
        xPosition = 1.sw - (widget.width ?? 0) / 2;
      }
      showHalf = true;
      LeooCodeUtilsJunPro.onSetState(this);
    });
  }

  void onHalfEnd() {
    showHalf = false;
    if (xPosition < .5.sw - 40.w) {
      xPosition = 12.w;
      if (widget.half && widget.width != 0) {
        onHalf(true);
      }
    } else {
      xPosition = x;
      if (widget.half && widget.width != 0) {
        onHalf(false);
      }
    }
    LeooCodeUtilsJunPro.onSetState(this);
  }

  @override
  void didUpdateWidget(LeooMoveStackWidgetJunPro oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (widget.update) {
      y = widget.yPosition;
      if (y == maxY) {
        if (yPosition + keyboardHeight > y) {
          yPosition = maxY! - keyboardHeight;
        }
      }
    }
  }

  @override
  Widget build(BuildContext context) {
    return Positioned(
        left: widget.right == true ? null : xPosition,
        right: widget.right == true ? 0 : null,
        top: yPosition,
        child: GestureDetector(
            onPanUpdate: (tapInfo) {
              FocusScope.of(context).unfocus();
              showHalf = false;
              timer?.cancel();
              xPosition += tapInfo.delta.dx;
              yPosition += tapInfo.delta.dy;
              if (xPosition < 12.w) {
                xPosition = 12.w;
              }
              if (xPosition > x) xPosition = x;
              if (yPosition <
                  (widget.chat ? 100.w : ScreenUtil().statusBarHeight + 50.w)) {
                yPosition =
                    (widget.chat ? 100.w : ScreenUtil().statusBarHeight + 50.w);
              }
              if (yPosition > y) yPosition = y;
              LeooCodeUtilsJunPro.onSetState(this);
            },
            onPanEnd: (DragEndDetails details) {
              onHalfEnd();
            },
            child: widget.child));
  }
}
