import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:leoo_junpro/leooflagjunpro/leooappjunpro/leoonetconfigjunpro.dart';

///右边出现停在左边指定位置
class LeooRightLeftWidgetJunPro extends StatefulWidget {
  final Widget child;
  final int position;
  final bool anim;

  const LeooRightLeftWidgetJunPro(this.child, this.position,
      {Key? key, this.anim = true})
      : super(key: key);

  @override
  _LeooRightLeftWidgetState createState() => _LeooRightLeftWidgetState();
}

class _LeooRightLeftWidgetState extends State<LeooRightLeftWidgetJunPro>
    with TickerProviderStateMixin {
  AnimationController? _controller;
  late Animation _animation;
  late Animation _animationFade;
  late int position;

  @override
  void initState() {
    position = widget.position;
    _controller = AnimationController(
        vsync: this, duration: const Duration(milliseconds: 500))
      ..addStatusListener((status) {
        if (status == AnimationStatus.completed) {
          Future.delayed(const Duration(milliseconds: 1500), () {
            if (position != 0) {
              try {
                _controller?.reset();
                position--;
                _controller?.forward();
              } catch (e) {
                LeooNetConfigJunPro.leooPrintLogJunPro(e);
              }
            } else {
              _controller?.dispose();
            }
          });
        }
      });
    _animation = Tween(begin: 50.w, end: 0).animate(_controller!);
    _animationFade = Tween(begin: 1.0, end: .0).animate(_controller!);
    if (widget.anim) _controller!.forward();
    super.initState();
  }

  @override
  void dispose() {
    _controller?.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return AnimatedBuilder(
        animation: _animation,
        builder: (BuildContext context, Widget? child) {
          double? value = _animation.value + (position - 1) * 50.w + 32.w;
          return Positioned(
              left: value,
              width: 60.w,
              height: 60.w,
              child: position != 0
                  ? child!
                  : AnimatedBuilder(
                      animation: _animationFade,
                      builder: (BuildContext context, Widget? child) {
                        return Opacity(
                            opacity: _animationFade.value, child: child);
                      },
                      child: child));
        },
        child: widget.child);
  }
}
