import 'package:flutter/material.dart';
import 'package:leoo_junpro/leooflagjunpro/leooappjunpro/leoonetconfigjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooutilsjunpro/leoocodeutilsjunpro.dart';

class LeooSpreadBtnWidgetJunPro extends StatefulWidget {
  const LeooSpreadBtnWidgetJunPro(
    this.child,
    this.radius,
    this.maxRadius,
    this.width,
    this.height, {
    Key? key,
    this.spreadColor = Colors.grey,
    this.duration = const Duration(milliseconds: 2000),
  }) : super(key: key);

  final Widget child;
  final double radius;
  final double maxRadius;
  final double width;
  final double height;
  final Color spreadColor;
  final Duration duration;

  @override
  _LeooSpreadBtnWidgetState createState() => _LeooSpreadBtnWidgetState();
}

class _LeooSpreadBtnWidgetState extends State<LeooSpreadBtnWidgetJunPro>
    with TickerProviderStateMixin {
  List<Widget> children = [];
  List<AnimationController>? controllers = [];
  bool load = false;
  bool starting = true;

  @override
  void initState() {
    children.add(widget.child);
    super.initState();
  }

  void start() async {
    while (starting) {
      try {
        if (ModalRoute.of(context)?.isCurrent == true) {
          if (controllers == null) {
            return;
          }
          AnimationController _animationController;
          Animation<double> _animation;
          _animationController =
              AnimationController(vsync: this, duration: widget.duration);
          _animation = CurvedAnimation(
              parent: _animationController, curve: Curves.linear);
          _animationController.addStatusListener((status) {
            if (status == AnimationStatus.completed) {
              _animationController.dispose();
              children.removeAt(0);
              controllers!.removeAt(0);
            }
          });
          controllers!.add(_animationController);
          _animationController.forward();
          children.add(LeooAnimatedSpreadJunPro(
              _animation,
              widget.radius,
              widget.maxRadius,
              widget.width,
              widget.height,
              widget.spreadColor));
          if (controllers == null) {
            return;
          }
          LeooCodeUtilsJunPro.onSetState(this);
        }
        await Future.delayed(
            Duration(milliseconds: widget.duration.inMilliseconds ~/ 2));
      } catch (e) {
        LeooNetConfigJunPro.leooPrintLogJunPro(e);
      }
    }
  }

  @override
  void dispose() {
    starting = false;
    for (var c in controllers!) {
      try {
        c.dispose();
      } catch (e) {
        LeooNetConfigJunPro.leooPrintLogJunPro(e);
      }
    }
    controllers = null;
    super.dispose();
  }

  @override
  void didUpdateWidget(covariant LeooSpreadBtnWidgetJunPro oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (children.last != widget.child) {
      children.last = widget.child;
    }
  }

  @override
  Widget build(BuildContext context) {
    if (!load) {
      load = true;
      start();
    }
    return Stack(children: children, alignment: Alignment.center);
  }
}

class LeooAnimatedSpreadJunPro extends AnimatedWidget {
  final Tween<double> _opacityTween = Tween(begin: 1, end: 0);
  final Tween<double> _radiusTween;
  final Color color;
  final double width;
  final double height;

  LeooAnimatedSpreadJunPro(Animation<double> animation, double radius,
      double maxRadius, this.width, this.height, this.color,
      {Key? key})
      : _radiusTween = Tween(begin: radius, end: maxRadius),
        super(key: key, listenable: animation);

  @override
  Widget build(BuildContext context) {
    final Animation<double> animation = listenable as Animation<double>;
    return ClipRRect(
      borderRadius: const BorderRadius.all(Radius.circular(50)),
      child: SizedBox(
        width: width + _radiusTween.evaluate(animation),
        height: height + _radiusTween.evaluate(animation),
        child: Opacity(
          opacity: _opacityTween.evaluate(animation),
          child: Container(color: color),
        ),
      ),
    );
  }
}
