import 'package:flutter/material.dart';
import 'package:leoo_junpro/leooflagjunpro/leooappjunpro/leoonetconfigjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooutilsjunpro/leoocodeutilsjunpro.dart';

class LeooSpreadWidgetJunPro extends StatefulWidget {
  const LeooSpreadWidgetJunPro(
    this.radius,
    this.maxRadius, {
    Key? key,
    this.child,
    this.spreadColor = Colors.grey,
    this.spreadColor2 = Colors.grey,
    this.duration = const Duration(milliseconds: 3000),
  }) : super(key: key);

  final Widget? child;
  final double? radius;
  final double? maxRadius;
  final Color? spreadColor;
  final Color? spreadColor2;
  final Duration duration;

  @override
  _LeooSpreadWidgetState createState() => _LeooSpreadWidgetState();
}

class _LeooSpreadWidgetState extends State<LeooSpreadWidgetJunPro>
    with TickerProviderStateMixin {
  List<Widget> children = [];
  List<AnimationController>? controllers = [];
  bool load = false;
  bool starting = true;

  @override
  void initState() {
    if (widget.child != null) {
      children.add(
        ClipOval(
          child: SizedBox(
              width: widget.radius, height: widget.radius, child: widget.child),
        ),
      );
    }
    super.initState();
  }

  void start() async {
    while (starting) {
      try {
        if (ModalRoute.of(context)?.isCurrent == true) {
          if (controllers == null) {
            return;
          }
          AnimationController _animationController;
          Animation<double> _animation;
          _animationController =
              AnimationController(vsync: this, duration: widget.duration);
          _animation = CurvedAnimation(
              parent: _animationController, curve: Curves.linear);
          _animationController.addStatusListener((status) {
            if (status == AnimationStatus.completed) {
              children.removeAt(0);
              controllers!.removeAt(0);
              _animationController.dispose();
            }
          });
          controllers!.add(_animationController);
          _animationController.forward();
          widget.child != null
              ? children.insert(
                  children.length - 1,
                  LeooAnimatedSpreadJunPro(
                      _animation,
                      widget.radius,
                      widget.maxRadius,
                      widget.spreadColor,
                      widget.spreadColor2))
              : children.add(LeooAnimatedSpreadJunPro(_animation, widget.radius,
                  widget.maxRadius, widget.spreadColor, widget.spreadColor2));
          LeooCodeUtilsJunPro.onSetState(this);
        }
        await Future.delayed(
            Duration(milliseconds: widget.duration.inMilliseconds ~/ 3));
      } catch (e) {
        LeooNetConfigJunPro.leooPrintLogJunPro(e);
      }
    }
  }

  @override
  void dispose() {
    starting = false;
    for (var c in controllers!) {
      c.dispose();
    }
    controllers = null;
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    if (!load) {
      load = true;
      start();
    }
    return Stack(children: children, alignment: Alignment.center);
  }
}

class LeooAnimatedSpreadJunPro extends AnimatedWidget {
  final Tween<double> _opacityTween = Tween(begin: 1, end: 0);
  final Tween<double> _radiusTween;
  final Color? color;
  final Color? color2;

  LeooAnimatedSpreadJunPro(Animation<double> animation, double? radius,
      double? maxRadius, this.color, this.color2,
      {Key? key})
      : _radiusTween = Tween(begin: radius, end: maxRadius),
        super(key: key, listenable: animation);

  @override
  Widget build(BuildContext context) {
    final Animation<double> animation = listenable as Animation<double>;
    return ClipOval(
      child: SizedBox(
        width: _radiusTween.evaluate(animation),
        height: _radiusTween.evaluate(animation),
        child: Opacity(
          opacity: _opacityTween.evaluate(animation),
          child: Container(
            decoration: BoxDecoration(
              gradient: LinearGradient(
                begin: Alignment.topCenter,
                end: Alignment.bottomCenter,
                colors: [color!, color2 ?? color!],
              ),
            ),
          ),
        ),
      ),
    );
  }
}
