import 'package:bubble_tab_indicator/bubble_tab_indicator.dart';
import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:leoo_junpro/leooflagjunpro/leoostringjunpro/leoostylejunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leoowidgetjunpro/leoocustomtabsjunpro.dart'
    as custom_tabs;
import 'package:leoo_junpro/leooflagjunpro/leoowidgetjunpro/leoomd2tabindicatorjunpro.dart';

///支持顶部和顶部的TabBar控件
///配合AutomaticKeepAliveClientMixin可以keep住
class LeooTabBarWidgetJunPro extends StatefulWidget {
  ///底部模式type
  static const int leooTabBottomJunPro = 1;

  ///顶部模式type
  static const int leooTaoTopJunPro = 2;
  static const int leooTaoTop2JunPro = 3;

  final int? type;
  final int position;

  final bool resizeToAvoidBottomInset;
  final bool isScrollable;
  final bool top;
  final Alignment? alignment;
  final EdgeInsets? margin;
  final EdgeInsets? padding;
  final EdgeInsets? labelPadding;

  final List<Widget> tabItems;
  final List<Widget> tabViews;

  final Color? backgroundColor;

  final Color? indicatorColor;
  final Color? labelColor;
  final Color? unselectedLabelColor;

  final Widget? title;

  final Widget? drawer;
  final Widget? endDrawer;

  final DrawerCallback? drawerChanged;
  final DrawerCallback? endDrawerChanged;

  final Widget? bottomBar;
  final bool showBar;

  final ValueChanged<int>? onPageChanged;
  final ValueChanged<int>? onPageCheck;

  final PageController? pageController;

  final Decoration? decoration;
  final double height;
  final double indicatorHeight;

  final List<Widget>? otherWidget;

  const LeooTabBarWidgetJunPro(this.tabItems, this.tabViews,
      {Key? key,
      this.type,
      this.backgroundColor,
      this.indicatorColor,
      this.title,
      this.drawer,
      this.endDrawer,
      this.bottomBar,
      this.resizeToAvoidBottomInset = false,
      this.onPageChanged,
      this.position = 0,
      this.onPageCheck,
      this.pageController,
      this.isScrollable = false,
      this.alignment,
      this.unselectedLabelColor,
      this.margin,
      this.padding,
      this.labelPadding,
      this.top = true,
      this.labelColor,
      this.decoration,
      this.showBar = true,
      this.otherWidget,
      this.height = .0,
      this.indicatorHeight = .0,
      this.drawerChanged,
      this.endDrawerChanged})
      : super(key: key);

  @override
  LeooTabBarWidgetState createState() => LeooTabBarWidgetState();
}

class LeooTabBarWidgetState extends State<LeooTabBarWidgetJunPro>
    with SingleTickerProviderStateMixin {
  int? _type;

  List<Widget>? _tabViews;

  Color? _indicatorColor;
  Color? _unselectedLabelColor;
  Color? _backgroundColor;

  Widget? _drawer;
  Widget? _endDrawer;

  DrawerCallback? drawerChanged;
  DrawerCallback? endDrawerChanged;

  ValueChanged<int>? _onPageChanged;
  ValueChanged<int>? _onPageCheck;

  late TabController _tabController;
  PageController? _pageController;
  var currentIndex = 0;

  @override
  void initState() {
    super.initState();
    currentIndex = widget.position;
    _type = widget.type;
    _tabViews = widget.tabViews;
    _indicatorColor = widget.indicatorColor;
    _unselectedLabelColor = widget.unselectedLabelColor;
    _drawer = widget.drawer;
    _endDrawer = widget.endDrawer;
    drawerChanged = widget.drawerChanged;
    endDrawerChanged = widget.endDrawerChanged;
    _onPageChanged = widget.onPageChanged;
    _onPageCheck = widget.onPageCheck;
    _backgroundColor = widget.backgroundColor;
    _pageController = widget.pageController;
    _tabController = TabController(
        vsync: this,
        length: widget.tabViews.length,
        initialIndex: currentIndex);
    _pageController ??= PageController(initialPage: currentIndex);
  }

  @override
  void didUpdateWidget(covariant LeooTabBarWidgetJunPro oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_backgroundColor != widget.backgroundColor) {
      _backgroundColor = widget.backgroundColor;
    }
    if (_drawer != widget.drawer) {
      _drawer = widget.drawer;
    }
    if (_endDrawer != widget.endDrawer) {
      _endDrawer = widget.endDrawer;
    }
  }

  ///整个页面dispose时，记得把控制器也dispose掉，释放内存
  @override
  void dispose() {
    _tabController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    if (_type == LeooTabBarWidgetJunPro.leooTaoTopJunPro) {
      ///顶部tab bar
      return Scaffold(
          resizeToAvoidBottomInset: widget.resizeToAvoidBottomInset,
          backgroundColor: _backgroundColor ?? Colors.transparent,
          appBar: !widget.showBar
              ? null
              : PreferredSize(
                  preferredSize: Size.fromHeight(
                      (widget.top ? ScreenUtil().statusBarHeight : 0) +
                          widget.height +
                          kToolbarHeight),
                  child: Container(
                    alignment: widget.alignment ?? Alignment.bottomCenter,
                    margin: widget.margin,
                    padding: widget.padding,
                    height: (widget.top ? ScreenUtil().statusBarHeight : 0) +
                        widget.height +
                        kToolbarHeight,
                    decoration: widget.decoration,
                    child: custom_tabs.TabBar(
                      controller: _tabController,
                      tabs: widget.tabItems,
                      labelPadding: widget.labelPadding ??
                          EdgeInsets.symmetric(horizontal: 12.w),
                      labelStyle: LeooTextJunPro.leooGetTextStyleJunPro(
                          20, FontWeight.bold),
                      unselectedLabelStyle:
                          LeooTextJunPro.leooGetTextStyleJunPro(
                              20, FontWeight.bold),
                      labelColor:
                          _indicatorColor ?? LeooColorsJunPro.leooAppJunPro,
                      unselectedLabelColor: _unselectedLabelColor ??
                          LeooColorsJunPro.leooTextHintJunPro,
                      isScrollable: widget.isScrollable,
                      physics: const BouncingScrollPhysics(),
                      indicatorSize: custom_tabs.TabBarIndicatorSize.label,
                      indicator: LeooMD2IndicatorJunPro(
                          4.w,
                          _indicatorColor ?? LeooColorsJunPro.leooAppJunPro,
                          MD2IndicatorSize.normal,
                          bottom: false),
                      onTap: (index) {
                        _onPageCheck?.call(index);
                        if ((currentIndex - index).abs() != 1) {
                          _pageController!.jumpToPage(index);
                        } else {
                          _pageController!.animateToPage(index,
                              duration: const Duration(milliseconds: 300),
                              curve: Curves.easeOutExpo);
                        }
                        currentIndex = index;
                      },
                    ),
                  ),
                ),
          body: PageView(
            controller: _pageController,
            children: _tabViews!,
            physics: const BouncingScrollPhysics(),
            onPageChanged: (index) {
              if (widget.showBar) {
                _tabController.animateTo(index);
              }
              _onPageChanged?.call(index);
              currentIndex = index;
            },
          ),
          bottomNavigationBar: widget.bottomBar);
    } else if (_type == LeooTabBarWidgetJunPro.leooTaoTop2JunPro) {
      ///顶部tab bar
      return Scaffold(
          resizeToAvoidBottomInset: widget.resizeToAvoidBottomInset,
          backgroundColor: _backgroundColor ?? Colors.transparent,
          appBar: !widget.showBar
              ? null
              : PreferredSize(
                  preferredSize: Size.fromHeight(30.w + widget.height),
                  child: Container(
                    alignment: widget.alignment ?? Alignment.center,
                    margin: widget.margin,
                    height: 30.w + widget.height,
                    child: Container(
                      height: 30.w + widget.height,
                      decoration: ShapeDecoration(
                          color: Colors.white.withOpacity(.2),
                          shape: const StadiumBorder()),
                      padding: EdgeInsets.only(left: 2.w, right: 2.w),
                      child: custom_tabs.TabBar(
                          controller: _tabController,
                          tabs: widget.tabItems,
                          labelPadding: widget.labelPadding ??
                              EdgeInsets.symmetric(horizontal: 16.w),
                          labelStyle: LeooTextJunPro.leooGetTextStyleJunPro(
                              20, FontWeight.bold),
                          unselectedLabelStyle:
                              LeooTextJunPro.leooGetTextStyleJunPro(
                                  20, FontWeight.bold),
                          labelColor: widget.labelColor ??
                              LeooColorsJunPro.leooAppJunPro,
                          unselectedLabelColor: _unselectedLabelColor ??
                              Colors.white.withOpacity(.8),
                          isScrollable: widget.isScrollable,
                          indicatorSize: custom_tabs.TabBarIndicatorSize.label,
                          indicator: BubbleTabIndicator(
                            indicatorHeight: widget.indicatorHeight,
                            indicatorColor: _indicatorColor ?? Colors.white,
                            tabBarIndicatorSize: TabBarIndicatorSize.label,
                            insets: const EdgeInsets.all(0),
                            padding: EdgeInsets.only(left: 15.w, right: 15.w),
                          ),
                          onTap: (index) {
                            _onPageCheck?.call(index);
                            if ((currentIndex - index).abs() != 1) {
                              _pageController!.jumpToPage(index);
                            } else {
                              _pageController!.animateToPage(index,
                                  duration: const Duration(milliseconds: 300),
                                  curve: Curves.easeOutExpo);
                            }
                            currentIndex = index;
                          }),
                    ),
                  )),
          body: PageView(
            controller: _pageController,
            children: _tabViews!,
            physics: const BouncingScrollPhysics(),
            onPageChanged: (index) {
              if (widget.showBar) {
                _tabController.animateTo(index);
              }
              _onPageChanged?.call(index);
              currentIndex = index;
            },
          ),
          bottomNavigationBar: widget.bottomBar);
    } else {
      var page = PageView(
        controller: _pageController,
        children: _tabViews!,
        allowImplicitScrolling: true,
        physics: const NeverScrollableScrollPhysics(),
        onPageChanged: (index) {
          if (widget.showBar) {
            _tabController.animateTo(index);
          }
          _onPageChanged?.call(index);
          currentIndex = index;
        },
      );
      if (widget.otherWidget != null) {
        widget.otherWidget!.insert(0, page);
      }
      return Scaffold(
        resizeToAvoidBottomInset: widget.resizeToAvoidBottomInset,
        backgroundColor: _backgroundColor ?? LeooColorsJunPro.leooBgJunPro,
        drawer: _drawer,
        endDrawer: _endDrawer,
        onDrawerChanged: drawerChanged,
        onEndDrawerChanged: endDrawerChanged ?? drawerChanged,
        body: widget.otherWidget == null
            ? page
            : Stack(
                alignment: Alignment.bottomCenter,
                children: widget.otherWidget!.map((e) => e).toList(),
              ),
        bottomNavigationBar: !widget.showBar
            ? null
            : Material(
                color: _backgroundColor ?? LeooColorsJunPro.leooBgJunPro,
                child: SafeArea(
                  maintainBottomViewPadding: true,
                  child: PreferredSize(
                    preferredSize:
                        const Size.fromHeight(kBottomNavigationBarHeight),
                    child: custom_tabs.TabBar(
                      //TabBar导航标签，底部导航放到Scaffold的bottomNavigationBar中
                      controller: _tabController,
                      //配置控制器
                      tabs: widget.tabItems,
                      indicatorColor: _indicatorColor,
                      indicatorWeight: .01,
                      labelStyle: LeooTextJunPro.leooGetTextStyleJunPro(14),
                      unselectedLabelStyle:
                          LeooTextJunPro.leooGetTextStyleJunPro(14),
                      labelColor:
                          widget.labelColor ?? LeooColorsJunPro.leooAppJunPro,
                      unselectedLabelColor: _unselectedLabelColor ??
                          LeooColorsJunPro.leooTextHintJunPro,
                      isScrollable: widget.isScrollable,
                      onTap: (index) {
                        _onPageCheck?.call(index);
                        if ((currentIndex - index).abs() != 1) {
                          _pageController!.jumpToPage(index);
                        } else {
                          _pageController!.animateToPage(index,
                              duration: const Duration(milliseconds: 300),
                              curve: Curves.easeOutExpo);
                        }
                        currentIndex = index;
                      },
                    ),
                  ),
                ),
              ),
      );
    }
  }
}
