import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:leoo_junpro/generated/l10n.dart';
import 'package:leoo_junpro/leooflagjunpro/leooappjunpro/leooappconfigjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooappjunpro/leoonetconfigjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leoostringjunpro/leoostylejunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooutilsjunpro/leoocodeutilsjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooutilsjunpro/leoocommonutilsjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooutilsjunpro/leoonavigatorutilsjunpro.dart';

class LeooBottomInputBarJunPro extends StatefulWidget {
  final LeooBottomInputBarDelegateJunPro delegate;
  final String hint;
  final bool gift;
  final bool video;
  final bool image;
  final TextStyle? textStyle;
  final TextStyle? hintStyle;
  final Color? color;
  final Color? borderColor;
  final TextEditingController controller;
  final bool autoFocus;
  final FocusNode? focusNode;
  final EdgeInsets? padding;
  final bool bottom;

  const LeooBottomInputBarJunPro(this.delegate, this.controller, this.hint,
      this.gift, this.video, this.image,
      {Key? key,
      this.textStyle,
      this.hintStyle,
      this.color,
      this.borderColor,
      this.autoFocus = false,
      this.padding,
      this.focusNode,
      this.bottom = true})
      : super(key: key);

  @override
  LeooBottomInputBarState createState() => LeooBottomInputBarState();
}

class LeooBottomInputBarState extends State<LeooBottomInputBarJunPro> {
  late TextField textField;
  late FocusNode focusNode;
  bool send = false;
  bool show = false; //软键盘是否打开
  late TextEditingController controller;
  String? hint;

  @override
  void initState() {
    super.initState();
    controller = widget.controller;
    hint = widget.hint;
    focusNode = widget.focusNode ?? FocusNode();
    send = controller.text.isEmpty == false;
    controller.addListener(() {
      bool empty = controller.text.isEmpty == false;
      if (empty != send) {
        send = empty;
        LeooCodeUtilsJunPro.onSetState(this);
        LeooNetConfigJunPro.leooPrintLogJunPro('输入框文本变更$empty');
      }
    });
    focusNode.addListener(() {
      _notifyInputStatusChanged(focusNode.hasFocus);
    });
    setField();
    if (widget.autoFocus) {
      focusNode.requestFocus();
    }
  }

  void setField() {
    textField = TextField(
        cursorColor: LeooColorsJunPro.leooAppJunPro,
        style: widget.textStyle ??
            LeooTextJunPro.leooGetTextStyleJunPro(16, FontWeight.bold),
        onSubmitted: _clickSendMessage,
        textInputAction: TextInputAction.send,
        controller: controller,
        decoration: InputDecoration(
            border: InputBorder.none,
            hintText: hint,
            hintStyle: widget.hintStyle ??
                LeooTextJunPro.leooGetTextStyleHintJunPro(16, FontWeight.bold)),
        focusNode: focusNode,
        minLines: 1,
        maxLines: 3,
        inputFormatters: [
          LengthLimitingTextInputFormatter(
              LeooAppConfigJunPro.leooTextSize200JunPro)
        ],
        onTap: () {
          LeooNetConfigJunPro.leooPrintLogJunPro('textField');
        });
  }

  void onSetState(bool show) {
    if (this.show != show) {
      this.show = show;
      LeooCodeUtilsJunPro.onSetState(this);
    }
  }

  void _clickSendMessage(String messageStr) {
    if (messageStr.isEmpty) {
      LeooNetConfigJunPro.leooPrintLogJunPro('不能为空');
      return;
    }
    if (messageStr.trim() == '') {
      LeooNetConfigJunPro.leooPrintLogJunPro('空格');
      LeooCommonUtilsJunPro.leooShowCustomToastJunPro(
          S.of(context).chatInputEmpty);
      controller.text = '';
      return;
    }
    widget.delegate.willInputSendText(messageStr);
  }

  photoMore() {
    LeooNetConfigJunPro.leooPrintLogJunPro('photoMore');
    if (focusNode.hasFocus) {
      focusNode.unfocus();
    }
    widget.delegate.didTapButton(2);
  }

  giftMore() {
    LeooNetConfigJunPro.leooPrintLogJunPro('giftMore');
    if (focusNode.hasFocus) {
      focusNode.unfocus();
    }
    widget.delegate.didTapButton(1);
  }

  videoMore() {
    LeooNetConfigJunPro.leooPrintLogJunPro('videoMore');
    if (focusNode.hasFocus) {
      focusNode.unfocus();
    }
    widget.delegate.didTapButton(3);
  }

  void _notifyInputStatusChanged(bool hasFocus) {
    widget.delegate.inputStatusDidChange(hasFocus);
  }

  @override
  void didUpdateWidget(covariant LeooBottomInputBarJunPro oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (hint != widget.hint) {
      hint = widget.hint;
      setField();
    }
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      color: widget.color ?? LeooColorsJunPro.leooBgJunPro,
      padding: widget.padding ??
          EdgeInsets.fromLTRB(
              15.w,
              5.w,
              15.w,
              (!show && widget.bottom ? ScreenUtil().bottomBarHeight : 0) +
                  5.w),
      child: Row(
        crossAxisAlignment: CrossAxisAlignment.end,
        children: <Widget>[
          if (widget.image)
            IconButton(
              padding: const EdgeInsets.all(0),
              iconSize: 40.w,
              icon: LeooCommonUtilsJunPro.leooGetLoadWidgetJunPro(
                  LeooIconsJunPro.leooChatImageJunPro,
                  width: 40.w,
                  height: 40.w),
              onPressed: LeooNavigatorUtilsJunPro.leooDebounceJunPro(
                  () => photoMore()),
            ),
          if (widget.gift)
            IconButton(
              padding: const EdgeInsets.all(0),
              iconSize: 40.w,
              icon: LeooCommonUtilsJunPro.leooLottieJunPro(
                LeooIconsJunPro.leooGiftJunPro,
                width: 40.w,
                height: 40.w,
              ),
              onPressed:
                  LeooNavigatorUtilsJunPro.leooDebounceJunPro(() => giftMore()),
            ),
          SizedBox(width: 5.w),
          Expanded(
            child: Container(
                decoration: BoxDecoration(
                  color: Colors.white.withOpacity(.1),
                  borderRadius: const BorderRadius.all(Radius.circular(16)),
                ),
                padding: EdgeInsets.only(left: 10.w, right: 10.w),
                child: textField),
          ),
          SizedBox(width: 5.w),
          widget.video && !send
              ? IconButton(
                  padding: const EdgeInsets.all(0),
                  iconSize: 40.w,
                  icon: LeooCommonUtilsJunPro.leooGetLoadWidgetJunPro(
                      LeooIconsJunPro.leooListVideoJunPro,
                      width: 40.w,
                      height: 40.w),
                  onPressed: LeooNavigatorUtilsJunPro.leooDebounceJunPro(
                      () => videoMore()),
                )
              : IconButton(
                  padding: const EdgeInsets.all(0),
                  iconSize: 40.w,
                  icon: LeooCommonUtilsJunPro.leooGetLoadWidgetJunPro(
                    send
                        ? LeooIconsJunPro.leooChatSendJunPro
                        : LeooIconsJunPro.leooChatSendNoJunPro,
                    width: 40.w,
                    height: 40.w,
                  ),
                  onPressed: LeooNavigatorUtilsJunPro.leooDebounceJunPro(() {
                    if (send) {
                      _clickSendMessage(controller.text);
                    }
                  }),
                ),
        ],
      ),
    );
  }
}

abstract class LeooBottomInputBarDelegateJunPro {
  ///输入工具栏状态发生变更
  void inputStatusDidChange(bool hasFocus);

  ///即将发送消息
  void willInputSendText(String text);

  ///点击事件 1-礼物 2-图片 3-通话
  void didTapButton(int type);
}
