import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:im_flutter_sdk/im_flutter_sdk.dart';
import 'package:leoo_junpro/leooflagjunpro/leooappjunpro/leoonetconfigjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leoostringjunpro/leoostylejunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooutilsjunpro/leoocodeutilsjunpro.dart';
import 'package:leoo_junpro/leooimpjunpro/leoocalljunpro/leooitemjunpro/leoomessageitemfactoryjunpro.dart';

class LeooConversationItemJunPro extends StatefulWidget {
  final EMMessage message;
  final LeooConversationItemDelegateJunPro? delegate;
  final bool showTime;
  final String? translator;
  final String? uid; //对方id
  final String? accessToken;

  const LeooConversationItemJunPro(this.delegate, this.message, this.showTime,
      this.translator, this.uid, this.accessToken,
      {Key? key})
      : super(key: key);

  @override
  State<StatefulWidget> createState() => _LeooConversationItemState();
}

class _LeooConversationItemState extends State<LeooConversationItemJunPro> {
  late EMMessage message;
  LeooConversationItemDelegateJunPro? delegate;
  late bool showTime;
  String? translator;
  String? uid;
  MessageStatusCallBack? callBack;

  void sendMessage() {
    if (message.direction == MessageDirection.SEND &&
        message.status == MessageStatus.CREATE) {
      message.setMessageStatusCallBack(callBack);
      try {
        EMClient.getInstance.chatManager.sendMessage(message);
      } catch (e) {
        LeooNetConfigJunPro.leooPrintLogJunPro(e);
      }
    }
  }

  @override
  void didUpdateWidget(LeooConversationItemJunPro oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (message != widget.message) {
      message = widget.message;
      showTime = widget.showTime;
      translator = widget.translator;
      sendMessage();
    }
  }

  @override
  void initState() {
    super.initState();
    message = widget.message;
    delegate = widget.delegate;
    showTime = widget.showTime;
    translator = widget.translator;
    uid = widget.uid;
    callBack = MessageStatusCallBack(
      onError: (error) {
        message.status = MessageStatus.FAIL;
        LeooCodeUtilsJunPro.onSetState(this);
      },
      onSuccess: () {
        message.status = MessageStatus.SUCCESS;
        LeooCodeUtilsJunPro.onSetState(this);
      },
    );
    sendMessage();
  }

  @override
  void dispose() {
    message.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      children: <Widget>[
        if (showTime)
          Center(
            child: Container(
              padding: EdgeInsets.all(15.w),
              child: Text(
                LeooCodeUtilsJunPro.leooGetNewsTimeStrJunPro(context,
                    DateTime.fromMillisecondsSinceEpoch(message.localTime)),
                style: LeooTextJunPro.leooGetTextStyleHintJunPro(14),
              ),
            ),
          ),
        buildMessageWidget(),
      ],
    );
  }

  void _onTapedMessage() {
    delegate?.didTapMessageItem(message);
  }

  void _onTapedMessageLong() {
    delegate?.didTapLongMessageItem(message);
  }

  Widget buildMessageWidget() {
    return Container(
      padding: EdgeInsets.fromLTRB(15.w, 7.5.w, 15.w, 7.5.w),
      alignment:
          uid == message.from ? Alignment.centerLeft : Alignment.centerRight,
      child: InkWell(
        onTap: () => _onTapedMessage(),
        onLongPress: () => _onTapedMessageLong(),
        child: Padding(
          padding: EdgeInsets.only(
              left: (uid == message.from ? 0 : 80).w,
              right: (uid == message.from ? 80 : 0).w),
          child: ClipRRect(
            borderRadius: uid == message.from
                ? const BorderRadius.only(
                    bottomLeft: Radius.circular(16),
                    bottomRight: Radius.circular(16),
                    topRight: Radius.circular(16))
                : const BorderRadius.only(
                    bottomLeft: Radius.circular(16),
                    topLeft: Radius.circular(16),
                    topRight: Radius.circular(16)),
            child: LeooMessageItemFactoryJunPro(
                message, uid == message.from, widget.accessToken,
                translator:
                    uid == message.from ? (translator ?? '') : translator),
          ),
        ),
      ),
    );
  }
}

abstract class LeooConversationItemDelegateJunPro {
  //点击消息
  void didTapMessageItem(EMMessage message);

  //点击消息长按
  void didTapLongMessageItem(EMMessage message);
}
