import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:flutter_spinkit/flutter_spinkit.dart';
import 'package:im_flutter_sdk/im_flutter_sdk.dart';
import 'package:leoo_junpro/generated/l10n.dart';
import 'package:leoo_junpro/leooflagjunpro/leooappjunpro/leooappconfigjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooappjunpro/leoonetconfigjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooentityjunpro/leoogiftentity.dart';
import 'package:leoo_junpro/leooflagjunpro/leoostringjunpro/leoostylejunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooutilsjunpro/leoocodeutilsjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooutilsjunpro/leoocommonutilsjunpro.dart';

class LeooConversationTextItemJunPro extends StatefulWidget {
  final EMMessage message;
  final String? translator;
  final String? meName;
  final String? userName;
  final bool send;

  const LeooConversationTextItemJunPro(
      this.message, this.translator, this.meName, this.userName, this.send,
      {Key? key})
      : super(key: key);

  @override
  State<StatefulWidget> createState() => _LeooConversationTextItemState();
}

class _LeooConversationTextItemState
    extends State<LeooConversationTextItemJunPro> {
  late EMMessage message;
  String? translator;
  String? meName;
  String? userName;
  late bool send;
  MessageStatusCallBack? callBack;

  void sendMessage() {
    if (message.direction == MessageDirection.SEND &&
        message.status == MessageStatus.CREATE) {
      message.setMessageStatusCallBack(callBack);
      try {
        EMClient.getInstance.chatManager.sendMessage(message);
      } catch (e) {
        LeooNetConfigJunPro.leooPrintLogJunPro(e);
      }
    }
  }

  @override
  void initState() {
    super.initState();
    message = widget.message;
    translator = widget.translator;
    meName = widget.meName;
    userName = widget.userName;
    send = widget.send;
    callBack = MessageStatusCallBack(
      onError: (error) {
        message.status = MessageStatus.FAIL;
        LeooCodeUtilsJunPro.onSetState(this);
      },
      onSuccess: () {
        message.status = MessageStatus.SUCCESS;
        LeooCodeUtilsJunPro.onSetState(this);
      },
    );
    sendMessage();
  }

  @override
  void dispose() {
    message.dispose();
    super.dispose();
  }

  @override
  void didUpdateWidget(LeooConversationTextItemJunPro oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (message != widget.message) {
      message = widget.message;
      translator = widget.translator;
      meName = widget.meName;
      userName = widget.userName;
      send = widget.send;
      sendMessage();
    }
  }

  @override
  Widget build(BuildContext context) {
    return Row(children: <Widget>[buildMessageWidget()]);
  }

  Widget buildMessageWidget() {
    if (message.body is EMTextMessageBody) {
      EMTextMessageBody msg = message.body as EMTextMessageBody;
      return Container(
        constraints: BoxConstraints(maxWidth: 1.sw - 160.w),
        padding: EdgeInsets.fromLTRB(8.w, 6.5.w, 8.w, 6.5.w),
        margin: EdgeInsets.only(left: 15.w),
        decoration: BoxDecoration(
          color: Colors.black.withOpacity(.3),
          borderRadius: BorderRadius.circular(15),
        ),
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            Text.rich(
                TextSpan(
                  text: message.to?.isEmpty == false
                      ? (send ? meName : userName)! + ': '
                      : '',
                  style: LeooTextJunPro.leooGetTextStyleAppJunPro(
                      16, FontWeight.bold),
                  children: <TextSpan>[
                    TextSpan(
                        text: send ? translator ?? msg.content : msg.content,
                        style: LeooTextJunPro.leooGetTextStyleJunPro(16))
                  ],
                ),
                softWrap: true),
            SizedBox(
                height:
                    translator != null && translator != msg.content ? 2.w : 0),
            translator != null && translator != msg.content
                ? translator?.isEmpty == false
                    ? Text.rich(
                        TextSpan(
                          text: (send ? meName : userName)! + ': ',
                          style: LeooTextJunPro.leooGetTextStyleAppJunPro(
                              16, FontWeight.bold),
                          children: <TextSpan>[
                            TextSpan(
                                text: (send ? msg.content : translator) ?? '',
                                style:
                                    LeooTextJunPro.leooGetTextStyleJunPro(16))
                          ],
                        ),
                        softWrap: true)
                    : SizedBox(
                        width: 20.w,
                        height: 20.w,
                        child: SpinKitFadingCircle(
                          color: LeooColorsJunPro.leooBtnBgJunPro,
                          size: 20.w,
                        ),
                      )
                : const SizedBox(),
          ],
        ),
      );
    } else if (message.body is EMCustomMessageBody) {
      EMCustomMessageBody body = message.body as EMCustomMessageBody;
      if (body.event == LeooAppConfigJunPro.leooGiftEventJunPro) {
        return giftV2MessageItem(body.params!);
      }
    }
    return Container();
  }

  Widget giftV2MessageItem(Map<String, String> map) {
    LeooGiftEntity msg = LeooGiftEntity.fromJson(map.cast<String, dynamic>());
    return Container(
      padding: EdgeInsets.fromLTRB(15.w, 5.w, 15.w, 5.w),
      decoration: BoxDecoration(
        borderRadius: const BorderRadius.only(
          topRight: Radius.circular(50),
          bottomRight: Radius.circular(50),
        ),
        gradient: LinearGradient(
          colors: [
            LeooColorsJunPro.leooAppJunPro.withOpacity(.1),
            LeooColorsJunPro.leooAppJunPro,
          ],
        ),
      ),
      child: Row(
        children: [
          Container(
            constraints: BoxConstraints(maxWidth: 150.w),
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                Text(
                  (send ? meName : userName) ?? '',
                  style: LeooTextJunPro.leooGetTextStyleJunPro(
                      16, FontWeight.bold),
                  maxLines: 1,
                ),
                Text(
                  S.of(context).sendTo((send ? userName : meName) ?? ''),
                  style: LeooTextJunPro.leooGetTextStyleJunPro(
                      20, FontWeight.bold),
                  maxLines: 1,
                ),
              ],
            ),
          ),
          LeooCommonUtilsJunPro.leooGetLoadWidgetJunPro(
            msg.imgUrl,
            width: 60.w,
            height: 60.w,
            color: false,
          )
        ],
      ),
    );
  }
}
