import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:flutter_spinkit/flutter_spinkit.dart';
import 'package:im_flutter_sdk/im_flutter_sdk.dart';
import 'package:leoo_junpro/leooflagjunpro/leooappjunpro/leooappconfigjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooentityjunpro/leoogiftentity.dart';
import 'package:leoo_junpro/leooflagjunpro/leoostringjunpro/leoostylejunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooutilsjunpro/leoocodeutilsjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooutilsjunpro/leoocommonutilsjunpro.dart';

import '../../../generated/l10n.dart';

class LeooMessageItemFactoryJunPro extends StatelessWidget {
  final EMMessage message;
  final String? translator;
  final bool otherSend; //对方发送的
  final String? accessToken;

  const LeooMessageItemFactoryJunPro(
      this.message, this.otherSend, this.accessToken,
      {Key? key, this.translator})
      : super(key: key);

  ///文本消息 item
  Widget textMessageItem() {
    EMTextMessageBody msg = message.body as EMTextMessageBody;
    return Column(
      mainAxisSize: MainAxisSize.min,
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        Text(otherSend ? msg.content : (translator ?? msg.content),
            style: LeooTextJunPro.leooGetTextStyleJunPro(16)),
        SizedBox(
            height: translator != null && translator != msg.content ? 5.w : 0),
        translator != null && translator != msg.content
            ? translator?.isEmpty == false
                ? Text((otherSend ? translator : msg.content) ?? '',
                    style: LeooTextJunPro.leooGetTextStyleHintJunPro(16))
                : SizedBox(
                    width: 20.w,
                    height: 20.w,
                    child: SpinKitFadingCircle(
                      color: LeooColorsJunPro.leooTextHintJunPro,
                      size: 20.w,
                    ),
                  )
            : const SizedBox(),
      ],
    );
  }

  ///文本消息 call
  Widget callMessageItem() {
    EMTextMessageBody msg = message.body as EMTextMessageBody;
    return Row(
      mainAxisSize: MainAxisSize.min,
      children: [
        LeooCommonUtilsJunPro.leooGetLoadWidgetJunPro(
          LeooIconsJunPro.leooInfoVideoJunPro,
          width: 20.w,
          height: 20.w,
        ),
        SizedBox(width: 5.w),
        Text(msg.content, style: LeooTextJunPro.leooGetTextStyleJunPro(16)),
      ],
    );
  }

  ///图片消息 item
  ///优先读缩略图，否则读本地路径图，否则读网络图
  Widget imageMessageItem() {
    EMImageMessageBody msg = message.body as EMImageMessageBody;
    Widget imageView;
    String path = LeooCodeUtilsJunPro.leooGetImageFileJunPro(msg.localPath);
    File file = File(path);
    if (file.existsSync() == true) {
      imageView = LeooCommonUtilsJunPro.leooGetLoadWidgetJunPro(
        path,
        width: 200.w,
        height: 300.w,
      );
    } else {
      imageView = LeooCommonUtilsJunPro.leooGetLoadWidgetJunPro(
        msg.remotePath,
        width: 200.w,
        height: 300.w,
        headers: {
          'Authorization': 'Bearer $accessToken',
          'share-secret': msg.secret ?? ''
        },
      );
    }
    String tag = '${message.msgId}imageMessageItem';
    return Container(
        constraints: BoxConstraints(
            minHeight: 100.w,
            minWidth: 100.w,
            maxWidth: 200.w,
            maxHeight: 300.w),
        child: Stack(
          alignment: Alignment.center,
          fit: StackFit.expand,
          children: [
            Hero(tag: tag, transitionOnUserGestures: true, child: imageView),
            if (message.direction == MessageDirection.SEND &&
                message.status == MessageStatus.FAIL)
              SizedBox(
                width: 40.w,
                height: 40.w,
                child: LeooCommonUtilsJunPro.leooGetLoadWidgetJunPro(
                    LeooIconsJunPro.leooMineBlacklistJunPro),
              ),
            if (message.direction == MessageDirection.SEND &&
                message.status == MessageStatus.PROGRESS)
              SpinKitFadingCircle(
                color: LeooColorsJunPro.leooAppJunPro,
                size: 40.w,
              ),
          ],
        ));
  }

  Widget giftV2MessageItem(BuildContext context, Map<String, String> map) {
    LeooGiftEntity msg = LeooGiftEntity.fromJson(map.cast<String, dynamic>());
    return Row(
      mainAxisSize: MainAxisSize.min,
      children: [
        Text(S.of(context).sendTo(''),
            style: LeooTextJunPro.leooGetTextStyleJunPro(16)),
        SizedBox(width: 5.w),
        LeooCommonUtilsJunPro.leooGetLoadWidgetJunPro(
          msg.imgUrl,
          width: 50.w,
          height: 50.w,
          color: false,
        ),
      ],
    );
  }

  Widget messageItem(BuildContext context) {
    if (message.body is EMTextMessageBody) {
      if (message.attributes?['customEvent'] ==
          LeooAppConfigJunPro.leooCallEventJunPro) {
        return Container(
            padding: EdgeInsets.fromLTRB(15.w, 12.w, 15.w, 12.w),
            color: _getMessageWidgetBGColor(),
            child: callMessageItem());
      }
      return Container(
          padding: EdgeInsets.fromLTRB(15.w, 12.w, 15.w, 12.w),
          color: _getMessageWidgetBGColor(),
          child: textMessageItem());
    } else if (message.body is EMImageMessageBody) {
      return imageMessageItem();
    } else if (message.body is EMCustomMessageBody) {
      EMCustomMessageBody body = message.body as EMCustomMessageBody;
      if (body.event == LeooAppConfigJunPro.leooImageEventJunPro) {
        String tag = '${message.msgId}imageMessageItem';
        return Container(
          constraints: BoxConstraints(
              minHeight: 100.w,
              minWidth: 100.w,
              maxWidth: 200.w,
              maxHeight: 300.w),
          child: Hero(
            tag: tag,
            transitionOnUserGestures: true,
            child: LeooCommonUtilsJunPro.leooGetLoadWidgetJunPro(
              body.params?['image'],
              width: 200.w,
              height: 300.w,
            ),
          ),
        );
      } else if (body.event == LeooAppConfigJunPro.leooGiftEventJunPro) {
        return Container(
            padding: EdgeInsets.fromLTRB(15.w, 0, 15.w, 0),
            color: _getMessageWidgetBGColor(),
            child: giftV2MessageItem(context, body.params!));
      }
    }
    return Container();
  }

  Color _getMessageWidgetBGColor() {
    return otherSend
        ? LeooColorsJunPro.leooAppJunPro
        : Colors.white.withOpacity(.1);
  }

  @override
  Widget build(BuildContext context) {
    return messageItem(context);
  }
}
