import 'dart:async';
import 'dart:io';

import 'package:agora_rtc_engine/rtc_engine.dart';
import 'package:agora_rtc_engine/rtc_local_view.dart' as rtc_local_view;
import 'package:agora_rtc_engine/rtc_remote_view.dart' as rtc_remote_view;
import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:leoo_junpro/leooflagjunpro/leooappjunpro/leoonetconfigjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leoodaojunpro/leoohostdaojunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooentityjunpro/leootokenentity.dart';
import 'package:leoo_junpro/leooflagjunpro/leooutilsjunpro/leoocodeutilsjunpro.dart';

///通话逻辑-2个主视图
class LeooVideoCallHostWidgetJunPro extends StatefulWidget {
  final String agoraAppKey; //声网key
  final int? userId; //对方uid
  final int? userHostId; //对方id
  final String name; //自己姓名
  final String portraitUri; //自己头像
  final int meUid; //自己uid
  final bool direct; //主动拨打下直接播放，还是等待数据返回拨号
  final double bottomBarHeight; //底部安全距离
  final LeooVideoCallHostDelegateJunPro? delegate; //回调方法
  final Widget? closeWidget; //关闭自己视频的占位

  const LeooVideoCallHostWidgetJunPro(Key key, this.agoraAppKey, this.userId,
      this.userHostId, this.name, this.portraitUri, this.meUid, this.direct,
      {this.delegate, this.closeWidget, this.bottomBarHeight = 0})
      : super(key: key);

  @override
  LeooVideoCallHostWidgetState createState() => LeooVideoCallHostWidgetState();
}

class LeooVideoCallHostWidgetState
    extends State<LeooVideoCallHostWidgetJunPro> {
  RtcEngine? rtcEngine;
  int? userUid; //对方uid-声网
  int? callId; //通话id
  String? channelId; //房间号
  String? callToken; //房间token
  bool isSoonMoney = false; //快没钱了
  bool? isConnect; //是否链接中，处理callStatus==0的时候文字显示问题
  bool isSendConnect = false; //是否发送邀请
  bool offline = false; //对方是否已经挂电话了
  bool isInit = false; //是否初始化成功
  //-2-余额不足状态 -1-未点击拨号(获取到数据显示待接听或拨号界面) 0-链接中 1-正在通话 2-已挂断 3-没钱已挂断
  int callStatus = -1;
  bool localVideoStatus = false; //是否本地推流成功
  bool remoteVideoStatus = false; //是否远端推流成功
  int currentRate = 0; //计时
  bool showTime = false; //显示计时器
  bool partyLeft = false; //对方是否挂断
  int currentDown = 0; //通话结束计时
  //网络质量检测
  int quality = 0;
  int qualityDown = 0;
  bool showVideoView = true; //显示界面其他按钮
  ///用户什么时候加入房间
  int userJoinedRate = 0;
  bool isLeave = true; //为加入房间不用离开房间-是否离开房间

  //ui控制
  bool closeCamera = false; //关闭摄像头
  bool closeMicrophone = false; //关闭麦克风
  double bottomHeight = 0; //键盘或者其他界面弹起时界面上移的距离

  @override
  void initState() {
    super.initState();
    Future(() {
      callBefore();
    });
  }

  @override
  void dispose() {
    rtcEngine?.disableLastmileTest();
    if (!isLeave) {
      isLeave = true;
      rtcEngine?.leaveChannel();
    }
    super.dispose();
  }

  void setBottomHeight(double bottomHeight) {
    this.bottomHeight = bottomHeight;
    LeooCodeUtilsJunPro.onSetState(this);
  }

  bool setShowVideoView() {
    showVideoView = !showVideoView;
    LeooCodeUtilsJunPro.onSetState(this);
    return showVideoView;
  }

  Future<bool> setCloseCamera() async {
    closeCamera = !closeCamera;
    await rtcEngine?.muteLocalVideoStream(closeCamera);
    LeooCodeUtilsJunPro.onSetState(this);
    return closeCamera;
  }

  Future<bool> setCloseMicrophone() async {
    closeMicrophone = !closeMicrophone;
    await rtcEngine?.enableLocalAudio(!closeMicrophone);
    await rtcEngine?.muteLocalAudioStream(closeMicrophone);
    return closeMicrophone;
  }

  void switchCamera() {
    rtcEngine?.switchCamera();
  }

  void setUserUid(int userUid) {
    this.userUid = userUid;
    LeooCodeUtilsJunPro.onSetState(this);
  }

  void setCallStatus(int callStatus) {
    this.callStatus = callStatus;
    if (callStatus == -1) {
      isSoonMoney = false;
      isConnect = null;
      isSendConnect = false;
      offline = false;
      isInit = false;
      localVideoStatus = false;
      remoteVideoStatus = false;
      showTime = false;
      partyLeft = false;
      showVideoView = true;
      isLeave = true;
      closeCamera = false;
      closeMicrophone = false;
    }
    LeooCodeUtilsJunPro.onSetState(this);
  }

  @override
  Widget build(BuildContext context) {
    return Stack(
      children: [
        if (userUid != null)
          Platform.isAndroid
              ? rtc_remote_view.TextureView(
                  uid: userUid!, channelId: channelId!)
              : rtc_remote_view.SurfaceView(
                  uid: userUid!, channelId: channelId!),
        if (widget.closeWidget != null && callStatus == 3) widget.closeWidget!,
        if ((callStatus == 1 || callStatus == 3) &&
            userUid != null &&
            showVideoView)
          Positioned(
            bottom: 90 + widget.bottomBarHeight + bottomHeight,
            right: 15,
            width: 120.w,
            height: 180.w,
            child: ClipRRect(
              borderRadius: BorderRadius.circular(8),
              child: Stack(
                fit: StackFit.expand,
                children: <Widget>[
                  Platform.isAndroid
                      ? const rtc_local_view.TextureView()
                      : const rtc_local_view.SurfaceView(),
                  if (widget.closeWidget != null && closeCamera)
                    widget.closeWidget ?? Container(),
                ],
              ),
            ),
          ),
      ],
    );
  }

  void userInHouseCheck() async {
    try {
      joinChannel();
    } catch (e) {
      LeooNetConfigJunPro.leooPrintLogJunPro(e);
      widget.delegate?.onShowToast(-1);
      willSendVideo(1);
      widget.delegate?.onNormalPop();
    }
  }

  void joinChannel() async {
    try {
      await rtcEngine?.disableLastmileTest();
      await rtcEngine?.joinChannel(callToken, channelId!, null, widget.meUid);
      widget.delegate?.onAnalyticsEvent('拨号接听时间$currentRate');
      isConnect = true;
      currentRate = 0;
      widget.delegate?.onCallStatus(callStatus);
      LeooCodeUtilsJunPro.onSetState(this);
    } catch (e) {
      printLog(e.toString());
    }
  }

  void callBefore() async {
    LeooTokenEntity? token = await widget.delegate?.callBefore();
    if (token == null) {
      return;
    }
    callId = token.talkingId;
    channelId = token.channelName;
    callToken = token.token;
    initRtcEngine();
  }

  void initRtcEngine() async {
    rtcEngine ??= await RtcEngine.create(widget.agoraAppKey);
    rtcEngine?.enableVideo();
    rtcEngine?.enableAudio();
    rtcEngine?.setChannelProfile(ChannelProfile.Communication);
    VideoEncoderConfiguration config = VideoEncoderConfiguration();
    config.orientationMode = VideoOutputOrientationMode.FixedPortrait;
    rtcEngine?.setVideoEncoderConfiguration(config);
    rtcEngine?.setBeautyEffectOptions(
      true,
      BeautyOptions(
        lighteningContrastLevel: LighteningContrastLevel.Normal,
        lighteningLevel: .6,
        smoothnessLevel: .5,
        rednessLevel: .1,
      ),
    );
    addAgoraEventHandlers();
    isInit = true;
    rtcEngine?.enableLastmileTest();
  }

  void addAgoraEventHandlers() {
    RtcEngineEventHandler handler = RtcEngineEventHandler();
    handler.tokenPrivilegeWillExpire = (String token) async {
      LeooTokenEntity? token =
          await widget.delegate?.callBefore(channelName: channelId);
      if (token == null) {
        return;
      }
      rtcEngine?.renewToken(token.token!);
    };
    handler.lastmileQuality = (NetworkQuality quality) {
      printLog('lastmileQuality: ${quality.index}');
      rtcEngine?.disableLastmileTest();
      if (quality == NetworkQuality.Down) {
        widget.delegate?.onShowToast(6);
      }
      joinChannel();
      willSendVideo(0);
    };
    handler.joinChannelSuccess = (String channel, int uid, int elapsed) {
      printLog('onJoinChannelSuccess: $channel, uid: $uid, elapsed: $elapsed');
      isLeave = false;
      if (uid == widget.userId && callStatus == -1) {
        callStatus = 0;
        widget.delegate?.onVideoSetState(callStatus);
      }
    };
    handler.userJoined = (int uid, int elapsed) {
      printLog('onUserJoined: uid: $uid, elapsed: $elapsed');
      userJoinedRate = DateTime.now().millisecondsSinceEpoch;
    };
    handler.localVideoStateChanged =
        (LocalVideoStreamState state, LocalVideoStreamError error) async {
      printLog('onLocalVideoStateChanged: $state $error');
      if (state == LocalVideoStreamState.Encoding) {
        localVideoStatus = true;
      }
    };
    handler.remoteVideoStateChanged = (int uid, VideoRemoteState state,
        VideoRemoteStateReason reason, int elapsed) {
      printLog('onRemoteVideoStateChanged: $uid $state $reason $elapsed');
      if (state == VideoRemoteState.Decoding && !remoteVideoStatus) {
        remoteVideoStatus = true;
        if (callStatus <= 0) {
          callStatus = 1;
          userUid = uid;
          widget.delegate?.onCallStatus(callStatus);
          callStart();
        }
      }
    };
    handler.networkQuality =
        (int uid, NetworkQuality txQuality, NetworkQuality rxQuality) {
      // printLog('onNetworkQuality: $uid $txQuality $rxQuality');
      if (callStatus != 1 || currentRate < 30) {
        return;
      }
      //下行网络
      if (rxQuality == NetworkQuality.Excellent ||
          rxQuality == NetworkQuality.Good ||
          rxQuality == NetworkQuality.Poor) {
        quality++;
        if (quality >= 3) {
          quality = 0;
          widget.delegate?.onShowNetText(null);
        }
      } else if (rxQuality == NetworkQuality.Unknown ||
          rxQuality == NetworkQuality.Bad ||
          rxQuality == NetworkQuality.VBad) {
        quality = 0;
        widget.delegate?.onShowNetText(uid == 0);
      }
    };
    handler.userOffline = (int uid, UserOfflineReason reason) {
      printLog('onUserOffline: $uid, reason: $reason');
      if (isSoonMoney && currentRate % 60 >= 55) {
        return;
      }
      if (offline) {
        return;
      }
      offline = true;
      widget.delegate
          ?.onRemoteUserLeft(reason == UserOfflineReason.Dropped ? 3 : 1);
    };
    handler.leaveChannel = (RtcStats stats) {
      printLog('onLeaveChannel');
      if (callStatus != 1) {
        return;
      }
      if (!offline) {
        widget.delegate?.onRemoteUserLeft(0);
      }
    };
    handler.error = (ErrorCode err) {
      printLog('onError: $err, index: ${err.index}');
      if (err.index < 1000 || err.index > 1400) {
        if (callStatus <= 0) {
          if (err == ErrorCode.ClientIsBannedByServer) {
            widget.delegate?.onShowToast(1);
          } else {
            widget.delegate?.onShowToast(2);
          }
          willSendVideo(1);
          widget.delegate?.onNormalPop();
        } else if (callStatus == 1) {
          widget.delegate?.onRemoteUserLeft(4);
        } else {
          widget.delegate?.onNormalPop(hangup: false);
        }
      }
    };
    handler.rtcStats = (RtcStats stats) {
      if (showTime && callStatus == 1) {
        currentCall();
        Future.delayed(const Duration(seconds: 1), () {
          currentCall();
        });
      }
    };
    rtcEngine?.setEventHandler(handler);
  }

  void currentCall() {
    currentRate++;
    // printLog('currentRate:$currentRate');
    if (callStatus == 1) {
      if (currentRate == 10 && !localVideoStatus) {
        widget.delegate?.onShowLocalVideo(true);
      }
      if (currentRate == 20 && !remoteVideoStatus) {
        widget.delegate?.onShowLocalVideo(false);
      }
      if (currentRate % 60 == 0) {
        if (isSoonMoney) {
          currentRate--;
        }
        widget.delegate?.onCalling();
      }
      if (currentDown > 0) {
        currentDown--;
        widget.delegate?.onCurrentRate(currentDown, down: true);
      }
      // printLog('currentDown:$currentDown');
    }
    widget.delegate?.onCurrentRate(currentRate);
  }

  Future callStart() async {
    currentRate = 0;
    showTime = true;
    LeooCodeUtilsJunPro.onSetState(this);
    widget.delegate?.onCalling();
  }

  void willSendVideo(int status) async {
    try {
      LeooNetConfigJunPro.leooPrintLogJunPro('willSendVideo: $status');
      if (status == 0) {
        if (callStatus == 0) {
          return;
        }
        LeooTokenEntity? token = await widget.delegate
            ?.callBefore(userId: widget.userHostId, channelName: channelId);
        if (token == null) {
          return;
        }
        LeooHostDaoJunPro.leooVideoCallJunPro(
            channelId,
            LeooNetConfigJunPro.leooAgoraAppKeyJunPro,
            widget.userId,
            widget.userHostId,
            token.token);
        isSendConnect = true;
      } else if (status == 1) {
        if (callStatus == 0) {
          return;
        }
        LeooHostDaoJunPro.leooVideoCloseJunPro(channelId, widget.userId);
      }
    } catch (e) {
      LeooNetConfigJunPro.leooPrintLogJunPro(e);
    }
  }

  void leaveChannel([bool keep = false]) {
    if (!isLeave) {
      isLeave = true;
      rtcEngine?.leaveChannel();
    }
    if (!keep && !offline) {
      widget.delegate?.onRemoteUserLeft(0);
    }
  }

  void printLog(String log) {
    LeooNetConfigJunPro.leooPrintLogJunPro(log);
  }
}

abstract class LeooVideoCallHostDelegateJunPro {
  //埋点
  void onAnalyticsEvent(String name);

  //本地推流或对方推流不成功提示及挂断
  void onShowLocalVideo(bool local);

  //通话状态维护
  void onCallStatus(int callStatus);

  //挂断电话
  ///0-自己挂断//1-对方正常挂断//2-没钱最后一分钟挂断//3-通话过程中对方异常//4-声网回调异常//5-后台挂断//6-匹配转正式没钱挂断
  void onRemoteUserLeft(int dropped, {bool toast = true});

  //toast提示
  ///-1-错误提示
  ///0-网络不好导致直接挂断-The network has been disconnected and the call is over!
  ///1-服务器封杀-Please try again after 60s!
  ///2-其他原因-Sorry, please try again later!
  ///3-对方不在线-Sorry, the other party is not online!
  ///5-对方取消通话-The other party has cancelled!
  ///6-当前网络可能无法通话-The other party has cancelled!
  void onShowToast(int type);

  //网络不好提示为null网络状态良好
  void onShowNetText(bool? me);

  //直接关闭页面false走挂断流程
  void onNormalPop({bool hangup = true, bool? delayed = false});

  //通话计时器-down=true倒计时通话计时器
  void onCurrentRate(int rate, {bool down = false});

  //通话检测
  Future<LeooTokenEntity?> callBefore({int? userId, String? channelName});

  //通话打点
  void onCalling([int? amount]);

  //刷新界面
  void onVideoSetState(int callStatus);
}
