import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:leoo_junpro/leooflagjunpro/leoostringjunpro/leoostylejunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooutilsjunpro/leoocodeutilsjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooutilsjunpro/leoocommonutilsjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooutilsjunpro/leoonavigatorutilsjunpro.dart';
import 'package:video_player/video_player.dart';

///播放视频的页面
class LeooPlayJunPro extends StatefulWidget {
  final String videoFileUrl;

  const LeooPlayJunPro(this.videoFileUrl, {Key? key}) : super(key: key);

  @override
  State<StatefulWidget> createState() => LeooPlayState();
}

class LeooPlayState extends State<LeooPlayJunPro> {
  late VideoPlayerController _controller;

  @override
  void initState() {
    super.initState();
    _controller = VideoPlayerController.file(File(widget.videoFileUrl));
    _controller.setLooping(true);
    _controller.initialize().then((value) {
      LeooCodeUtilsJunPro.onSetState(this);
      _controller.play();
    });
  }

  @override
  void dispose() {
    _controller.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      resizeToAvoidBottomInset: false,
      body: Stack(
        fit: StackFit.expand,
        children: [
          GestureDetector(
            behavior: HitTestBehavior.opaque,
            onTap: () {
              if (_controller.value.isInitialized == true) {
                if (_controller.value.isPlaying != false) {
                  _controller.pause();
                } else {
                  _controller.play();
                }
                LeooCodeUtilsJunPro.onSetState(this);
              }
            },
            child: Stack(
              alignment: Alignment.center,
              children: <Widget>[
                ClipRect(
                  child: _controller.value.isInitialized == true
                      ? Transform.scale(
                          scale: _controller.value.aspectRatio > 1
                              ? 1
                              : (_controller.value.aspectRatio <
                                      MediaQuery.of(context).size.aspectRatio
                                  ? MediaQuery.of(context).size.aspectRatio /
                                      _controller.value.aspectRatio
                                  : _controller.value.aspectRatio /
                                      MediaQuery.of(context).size.aspectRatio),
                          child: Center(
                            child: AspectRatio(
                              aspectRatio: _controller.value.aspectRatio,
                              child: VideoPlayer(_controller),
                            ),
                          ),
                        )
                      : Container(
                          width: 80.w,
                          height: 80.w,
                          padding: EdgeInsets.all(20.w),
                          child: CircularProgressIndicator(
                            backgroundColor: LeooColorsJunPro.leooBtnBgJunPro,
                            valueColor: AlwaysStoppedAnimation<Color>(
                              LeooColorsJunPro.leooAppJunPro,
                            ),
                          ),
                        ),
                ),
                Positioned(
                  bottom: ScreenUtil().bottomBarHeight + 30.w,
                  child: SizedBox(
                    width: 1.sw,
                    height: 2.w,
                    child: _controller.value.isInitialized == true
                        ? VideoProgressIndicator(_controller,
                            allowScrubbing: false,
                            colors: const VideoProgressColors(
                                playedColor: Colors.white,
                                bufferedColor: Colors.white60,
                                backgroundColor: Colors.white24),
                            padding: const EdgeInsets.all(0))
                        : const LinearProgressIndicator(
                            backgroundColor: Colors.white24,
                            valueColor:
                                AlwaysStoppedAnimation<Color>(Colors.white),
                          ),
                  ),
                ),
                if (_controller.value.isInitialized == true &&
                    _controller.value.isPlaying == false)
                  LeooCommonUtilsJunPro.leooGetLoadWidgetJunPro(
                    LeooIconsJunPro.leooVideoPlayJunPro,
                    width: 60.w,
                    height: 60.w,
                  ),
              ],
            ),
          ),
          Positioned(
            top: ScreenUtil().statusBarHeight + 4.w,
            left: 4.w,
            child: IconButton(
              iconSize: 40.w,
              padding: const EdgeInsets.all(0),
              icon: LeooCommonUtilsJunPro.leooGetLoadWidgetJunPro(
                  LeooIconsJunPro.leooTopBackJunPro,
                  width: 40.w,
                  height: 40.w),
              onPressed: LeooNavigatorUtilsJunPro.leooDebounceJunPro(() {
                _controller.pause();
                Navigator.pop(context);
              }),
            ),
          )
        ],
      ),
    );
  }
}
