import 'dart:async';
import 'dart:core';

import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:leoo_junpro/generated/l10n.dart';
import 'package:leoo_junpro/leooflagjunpro/leooappjunpro/leooappconfigjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leoodaojunpro/leoomaindaojunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooentityjunpro/leoovideolistentity.dart';
import 'package:leoo_junpro/leooflagjunpro/leooeventjunpro/leooindexjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooeventjunpro/leoomaineventjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooproviderjunpro/leoouserproviderjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooutilsjunpro/leoocodeutilsjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooutilsjunpro/leoocommonutilsjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooutilsjunpro/leoowidgetutilsjunpro.dart';
import 'package:provider/provider.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';

///列表zb
class LeooVideoItemJunPro extends StatefulWidget {
  final int type;

  const LeooVideoItemJunPro(this.type, {Key? key}) : super(key: key);

  @override
  _LeooVideoItemState createState() => _LeooVideoItemState();
}

class _LeooVideoItemState extends State<LeooVideoItemJunPro>
    with AutomaticKeepAliveClientMixin<LeooVideoItemJunPro> {
  @override
  bool get wantKeepAlive => true;

  final RefreshController _refreshController = RefreshController();
  int page = 1;
  List<LeooVideoListEntity> allList = [];
  int status = LeooCommonUtilsJunPro.leooPageStatusLoadingJunPro;
  StreamSubscription? subscription;
  StreamSubscription? subscriptionUnLike;

  @override
  void initState() {
    super.initState();
    if (widget.type == 1 || widget.type == 2) {
      subscriptionUnLike =
          eventBus.on<LeooVideoInfoUnLikeEventJunPro>().listen((event) {
        status = LeooCommonUtilsJunPro.leooPageStatusLoadingJunPro;
        LeooCodeUtilsJunPro.onSetState(this);
        page = 1;
        getList();
      });
    }
    if (widget.type == 2) {
      subscription =
          eventBus.on<LeooVideoInfoFollowEventJunPro>().listen((event) {
        status = LeooCommonUtilsJunPro.leooPageStatusLoadingJunPro;
        LeooCodeUtilsJunPro.onSetState(this);
        page = 1;
        getList();
      });
    } else if (widget.type == 3) {
      subscription = eventBus.on<LeooVideoListMyEventJunPro>().listen((event) {
        status = LeooCommonUtilsJunPro.leooPageStatusLoadingJunPro;
        LeooCodeUtilsJunPro.onSetState(this);
        page = 1;
        getList();
      });
    }
    getList();
  }

  @override
  void dispose() {
    subscription?.cancel();
    subscriptionUnLike?.cancel();
    super.dispose();
  }

  void getList() {
    LeooMainDaoJunPro.leooVideoListJunPro(widget.type, page).then((res) async {
      if (res.result) {
        if (page == 1) {
          allList.clear();
        }
        allList.addAll(res.data);
        if (allList.isEmpty) {
          status = LeooCommonUtilsJunPro.leooPageStatusEmptyJunPro;
        } else {
          status = LeooCommonUtilsJunPro.leooPageStatusSuccessJunPro;
        }
        LeooCodeUtilsJunPro.onSetState(this);
        if (page == 1) {
          _refreshController.refreshCompleted();
          _refreshController.loadComplete();
        } else if (res.data.isEmpty) {
          _refreshController.loadNoData();
        } else {
          _refreshController.loadComplete();
        }
      } else {
        if (allList.isEmpty) {
          status = LeooCommonUtilsJunPro.leooPageStatusErrorJunPro;
          LeooCodeUtilsJunPro.onSetState(this);
        } else if (page == 1) {
          _refreshController.refreshFailed();
        } else {
          _refreshController.loadFailed();
        }
        LeooCommonUtilsJunPro.leooShowErrorToastJunPro(res);
      }
    });
  }

  @override
  Widget build(BuildContext context) {
    super.build(context);
    return Consumer<LeooUserProviderJunPro>(
        builder: (_, LeooUserProviderJunPro userProvider, __) {
      return status != LeooCommonUtilsJunPro.leooPageStatusSuccessJunPro
          ? LeooCommonUtilsJunPro.leooCheckPageStatusJunPro(
              context,
              status,
              onTap: () {
                status = LeooCommonUtilsJunPro.leooPageStatusLoadingJunPro;
                LeooCodeUtilsJunPro.onSetState(this);
                getList();
              },
              empty: S.of(context).emptyVideo,
            )
          : SmartRefresher(
              controller: _refreshController,
              header: const ClassicHeader(),
              footer: const ClassicFooter(),
              enablePullUp:
                  allList.length * 3 >= LeooAppConfigJunPro.leooPageSizeJunPro,
              onRefresh: () {
                page = 1;
                getList();
              },
              onLoading: () {
                page++;
                getList();
              },
              child: GridView.builder(
                  physics: const BouncingScrollPhysics(),
                  itemCount: allList.length,
                  padding: EdgeInsets.all(5.w),
                  gridDelegate: SliverGridDelegateWithFixedCrossAxisCount(
                      crossAxisCount: 2,
                      childAspectRatio: .64,
                      mainAxisSpacing: 5.w,
                      crossAxisSpacing: 5.w),
                  itemBuilder: (BuildContext itemContext, int index) {
                    return LeooWidgetUtilsJunPro
                        .leooGetVideoListChildWidgetJunPro(context, widget.type,
                            page, allList[index], index, allList);
                  }),
            );
    });
  }
}
