import 'dart:async';
import 'dart:core';

import 'package:agora_rtm/agora_rtm.dart';
import 'package:collection/collection.dart';
import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:leoo_junpro/generated/l10n.dart';
import 'package:leoo_junpro/leooflagjunpro/leooappjunpro/leooappconfigjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leoodaojunpro/leoomaindaojunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooentityjunpro/leoolanguageentity.dart';
import 'package:leoo_junpro/leooflagjunpro/leooentityjunpro/leoortmentity.dart';
import 'package:leoo_junpro/leooflagjunpro/leooentityjunpro/leoouserentity.dart';
import 'package:leoo_junpro/leooflagjunpro/leooproviderjunpro/leoouserproviderjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leoostringjunpro/leoostylejunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooutilsjunpro/leoocodeutilsjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooutilsjunpro/leoocommonutilsjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooutilsjunpro/leoodialogutilsjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooutilsjunpro/leoonavigatorutilsjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooutilsjunpro/leoovideoutilsjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooutilsjunpro/leoowidgetutilsjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leoowidgetjunpro/leoobuttonwidgetjunpro.dart';
import 'package:provider/provider.dart';
import 'package:video_player/video_player.dart';

///Random详细
class LeooMatInfoJunPro extends StatefulWidget {
  static bool leooOpenJunPro = false;
  final int? matchingId;
  final LeooUserEntity infoEntity;
  final String? videoUrl;

  const LeooMatInfoJunPro(this.matchingId, this.infoEntity,
      {Key? key, this.videoUrl})
      : super(key: key);

  @override
  _LeooMatchInfoState createState() => _LeooMatchInfoState();
}

class _LeooMatchInfoState extends State<LeooMatInfoJunPro>
    with OnRtmPluginDelegate {
  GlobalKey<LeooButtonWidgetState> textKeyDown = GlobalKey();
  VideoPlayerController? _controller;

  int currentDown = LeooAppConfigJunPro.leooMatchInfoTimeJunPro;
  Timer? beforeTimer;
  late double aspectRatio;
  List<LeooLanguageEntity> tagList = [];
  bool agreeJoin = false;

  @override
  void initState() {
    super.initState();
    LeooMatInfoJunPro.leooOpenJunPro = true;
    LeooVideoUtilsJunPro.instance.rtmPluginDelegateList.add(this);
    aspectRatio = (1.sw - 40.w) / 400.w;
    if (widget.videoUrl?.isEmpty == false) {
      _controller = VideoPlayerController.network(widget.videoUrl!);
      _controller!.setVolume(.0);
      _controller!.setLooping(true);
      _controller!.initialize().then((value) {
        LeooCodeUtilsJunPro.onSetState(this);
        _controller?.play();
      });
    }
    beforeTimer = Timer.periodic(const Duration(seconds: 1), (timer) {
      currentDown--;
      textKeyDown.currentState?.onSetState(agreeJoin
          ? S.of(context).matchApproved(currentDown)
          : S.of(context).matchAgree(currentDown));
      if (currentDown == 0) {
        timer.cancel();
        Navigator.pop(context, 3);
      }
    });
    getTagList();
  }

  void getTagList() {
    LeooMainDaoJunPro.leooGetTagListJunPro().then((res) async {
      if (res.result) {
        tagList = res.data;
        LeooCodeUtilsJunPro.onSetState(this);
      } else {
        LeooCommonUtilsJunPro.leooShowErrorToastJunPro(res);
      }
    });
  }

  @override
  void dispose() {
    LeooMatInfoJunPro.leooOpenJunPro = false;
    LeooVideoUtilsJunPro.instance.rtmPluginDelegateList.remove(this);
    beforeTimer?.cancel();
    _controller?.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Consumer<LeooUserProviderJunPro>(
        builder: (_, LeooUserProviderJunPro userProvider, __) {
      return Center(
        child: Column(
          mainAxisSize: MainAxisSize.min,
          children: [
            SizedBox(height: 20.w),
            Stack(
              alignment: Alignment.bottomCenter,
              children: [
                Container(
                  width: 110.w,
                  height: 10.w,
                  decoration: ShapeDecoration(
                    shape: StadiumBorder(),
                    color: LeooColorsJunPro.leooAppJunPro,
                  ),
                ),
                Text(
                  S.of(context).matchTip,
                  style: LeooTextJunPro.leooGetTextStyleJunPro(
                      24, FontWeight.bold),
                ),
              ],
            ),
            SizedBox(height: 30.w),
            Expanded(
              child: SizedBox(
                width: 1.sw - 30.w,
                child: Stack(
                  fit: StackFit.expand,
                  children: [
                    SizedBox(
                      width: 1.sw,
                      height: 1.sh,
                      child: ClipRRect(
                        borderRadius: BorderRadius.circular(20),
                        child: _controller?.value.isInitialized == true
                            ? Transform.scale(
                                scale: _controller!.value.aspectRatio > 1
                                    ? 1
                                    : (_controller!.value.aspectRatio <
                                            aspectRatio
                                        ? aspectRatio /
                                            _controller!.value.aspectRatio
                                        : _controller!.value.aspectRatio /
                                            aspectRatio),
                                child: Center(
                                  child: AspectRatio(
                                    aspectRatio: _controller!.value.aspectRatio,
                                    child: VideoPlayer(_controller!),
                                  ),
                                ),
                              )
                            : LeooCommonUtilsJunPro.leooGetLoadWidgetJunPro(
                                widget.infoEntity.header,
                                load: LeooIconsJunPro.leooLoadHeadJunPro,
                              ),
                      ),
                    ),
                    Positioned(
                      left: 15.w,
                      right: 15.w,
                      bottom: 15.w,
                      child: Column(
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: [
                          Row(
                            children: [
                              Text(
                                widget.infoEntity.getNameAge(),
                                style: LeooTextJunPro.leooGetTextStyleJunPro(
                                    20, FontWeight.bold),
                              ),
                              SizedBox(width: 5.w),
                              if ((widget.infoEntity.gender ?? 0) != 0)
                                LeooCommonUtilsJunPro.leooGetLoadWidgetJunPro(
                                  widget.infoEntity.gender == 1
                                      ? LeooIconsJunPro.leooInfoMaleJunPro
                                      : LeooIconsJunPro.leooInfoFemaleJunPro,
                                  width: 18.w,
                                  height: 18.w,
                                ),
                            ],
                          ),
                          SizedBox(height: 5.w),
                          if (widget.infoEntity.country?.isEmpty == false)
                            LeooWidgetUtilsJunPro.leooGetCountryWidgetJunPro(
                                context, widget.infoEntity.country,
                                item: false),
                          if (widget.infoEntity.tagIds?.isEmpty == false)
                            SizedBox(height: 5.w),
                          if (widget.infoEntity.tagIds?.isEmpty == false)
                            Wrap(
                              spacing: 5.w,
                              runSpacing: 5.w,
                              children: widget.infoEntity.tagIds!.map(
                                (item) {
                                  LeooLanguageEntity? tag =
                                      tagList.firstWhereOrNull(
                                          (element) => element.id == item);
                                  if (tag != null) {
                                    Color color = LeooColorsJunPro
                                        .leooGetRandomColorJunPro();
                                    return Container(
                                      padding: EdgeInsets.fromLTRB(
                                          8.w, 5.w, 8.w, 5.w),
                                      decoration: ShapeDecoration(
                                          color: color,
                                          shape: const StadiumBorder()),
                                      child: Text(tag.enName ?? '',
                                          style: LeooTextJunPro
                                              .leooGetTextStyleJunPro(14)),
                                    );
                                  }
                                  return Container();
                                },
                              ).toList(),
                            ),
                          if (widget.infoEntity.description?.isEmpty == false)
                            SizedBox(height: 5.w),
                          if (widget.infoEntity.description?.isEmpty == false)
                            Text(
                              widget.infoEntity.description ?? '',
                              style: LeooTextJunPro.leooGetTextStyleJunPro(
                                  16, FontWeight.bold),
                            ),
                        ],
                      ),
                    ),
                  ],
                ),
              ),
            ),
            SizedBox(height: 20.w),
            Row(
              children: [
                SizedBox(width: 20.w),
                Expanded(
                  child: LeooButtonWidgetJunPro(
                    agreeJoin
                        ? S.of(context).matchApproved(currentDown)
                        : S.of(context).matchAgree(currentDown),
                    LeooTextJunPro.leooGetTextStyleJunPro(20, FontWeight.bold),
                    key: textKeyDown,
                    color: LeooColorsJunPro.leooAppJunPro,
                    colorRight: LeooColorsJunPro.leooApp2JunPro,
                    shape: const RoundedRectangleBorder(
                      borderRadius: BorderRadius.all(Radius.circular(20)),
                    ),
                    onPressed: LeooNavigatorUtilsJunPro.leooDebounceJunPro(() {
                      if (!agreeJoin) {
                        LeooDialogUtilsJunPro.instance
                            .showLoadingDialog(context);
                        LeooMainDaoJunPro.leooOperateJunPro(
                                widget.matchingId, 1)
                            .then((res) {
                          Navigator.pop(context);
                          if (res.result) {
                            if (res.data.result == 0) {
                              agreeJoin = true;
                              textKeyDown.currentState?.onSetState(agreeJoin
                                  ? S.of(context).matchApproved(currentDown)
                                  : S.of(context).matchAgree(currentDown));
                            } else if (res.data.result == 1) {
                              LeooVideoUtilsJunPro.instance
                                  .requestVideoPermission(
                                      context, widget.infoEntity.id,
                                      matchingId: widget.matchingId,
                                      infoEntity: widget.infoEntity,
                                      callId: res.data.talkingId,
                                      channelId: res.data.channelName);
                            } else if (res.data.result == 2) {
                              beforeTimer?.cancel();
                              Navigator.pop(context, 4);
                            } else {
                              LeooCommonUtilsJunPro.leooShowErrorToastJunPro(
                                  res);
                            }
                          } else {
                            LeooCommonUtilsJunPro.leooShowErrorToastJunPro(res);
                          }
                        });
                      }
                    }),
                  ),
                ),
                SizedBox(width: 15.w),
                IconButton(
                  iconSize: 50.w,
                  padding: const EdgeInsets.all(0),
                  icon: LeooCommonUtilsJunPro.leooGetLoadWidgetJunPro(
                      LeooIconsJunPro.leooPopClose3JunPro,
                      width: 50.w,
                      height: 50.w),
                  onPressed: LeooNavigatorUtilsJunPro.leooDebounceJunPro(() {
                    beforeTimer?.cancel();
                    Navigator.pop(context, 2);
                  }),
                ),
                SizedBox(width: 20.w),
              ],
            ),
            SizedBox(height: 20.w),
          ],
        ),
      );
    });
  }

  @override
  void onRtmMessageReceived(LeooRtmEntity rtmEntity) {
    if (currentDown == 0) {
      return;
    }
    if (rtmEntity.type == 2) {
      LeooVideoUtilsJunPro.instance.requestVideoPermission(
        context,
        rtmEntity.matchingUserId,
        matchingId: rtmEntity.matchingId,
        infoEntity: widget.infoEntity,
        callId: rtmEntity.talkingId,
        channelId: rtmEntity.channelName,
      );
    } else if (rtmEntity.type == 4) {
      beforeTimer?.cancel();
      Navigator.pop(context, 4);
    }
  }

  @override
  void onRemoteInvitationCanceled(AgoraRtmRemoteInvitation invite) {}
}
