import 'package:flutter/material.dart';
import 'package:flutter_picker/flutter_picker.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:leoo_junpro/generated/l10n.dart';
import 'package:leoo_junpro/leooflagjunpro/leoodaojunpro/leoomaindaojunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooentityjunpro/leoodetailentity.dart';
import 'package:leoo_junpro/leooflagjunpro/leoostringjunpro/leoostylejunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooutilsjunpro/leoocodeutilsjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooutilsjunpro/leoocommonutilsjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooutilsjunpro/leoonavigatorutilsjunpro.dart';
import 'package:leoo_junpro/leooflagjunpro/leooutilsjunpro/leoowidgetutilsjunpro.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';

///数据统计详细
class LeooDetailJunPro extends StatefulWidget {
  const LeooDetailJunPro({Key? key}) : super(key: key);

  @override
  _LeooDetailState createState() => _LeooDetailState();
}

class _LeooDetailState extends State<LeooDetailJunPro> {
  final RefreshController _refreshController = RefreshController();
  List<LeooDetailEntity> detailList = [];
  int page = 1;
  int status = LeooCommonUtilsJunPro.leooPageStatusLoadingJunPro;
  String? month;
  bool haveMore = false;

  void getDetailList() {
    LeooMainDaoJunPro.leooListCommisionJunPro(page, month).then((res) {
      if (res.result) {
        if (page == 1) {
          detailList = res.data;
          for (var element in detailList) {
            haveMore = element.listDetail.isNotEmpty;
          }
        } else {
          for (var data in res.data) {
            List<LeooDetailEntity> haveItem = [];
            for (var element in detailList) {
              if (element.month == data.month) {
                element.listDetail.addAll(data.listDetail);
              } else {
                haveItem.add(data);
              }
              haveMore = data.listDetail.isNotEmpty;
            }
            if (haveItem.isNotEmpty) {
              detailList.addAll(haveItem);
            }
          }
        }
        if (detailList.isEmpty) {
          status = LeooCommonUtilsJunPro.leooPageStatusEmptyJunPro;
        } else {
          status = LeooCommonUtilsJunPro.leooPageStatusSuccessJunPro;
        }
        LeooCodeUtilsJunPro.onSetState(this);
        if (page == 1) {
          _refreshController.refreshCompleted();
          _refreshController.loadComplete();
        } else if (res.data.isEmpty) {
          _refreshController.loadNoData();
        } else {
          _refreshController.loadComplete();
        }
      } else {
        if (detailList.isEmpty) {
          status = LeooCommonUtilsJunPro.leooPageStatusErrorJunPro;
          LeooCodeUtilsJunPro.onSetState(this);
        } else if (page == 1) {
          _refreshController.refreshFailed();
        } else {
          _refreshController.loadFailed();
        }
        LeooCommonUtilsJunPro.leooShowErrorToastJunPro(res);
      }
    });
  }

  @override
  void initState() {
    super.initState();
    getDetailList();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      resizeToAvoidBottomInset: false,
      body: SafeArea(
        maintainBottomViewPadding: true,
        child: Column(
          children: <Widget>[
            LeooWidgetUtilsJunPro.leooGetAppBarWidgetJunPro(
              context,
              S.of(context).detail,
              rightChild: InkWell(
                child: Padding(
                  padding: EdgeInsets.all(6.w),
                  child: Row(
                    children: [
                      Text(
                        S.of(context).detailFilter,
                        style: LeooTextJunPro.leooGetTextStyleJunPro(
                            16, FontWeight.bold),
                      ),
                      SizedBox(width: 5.w),
                      LeooCommonUtilsJunPro.leooGetLoadWidgetJunPro(
                        LeooIconsJunPro.leooItemAddJunPro,
                        width: 20.w,
                        height: 20.w,
                      ),
                    ],
                  ),
                ),
                onTap: LeooNavigatorUtilsJunPro.leooDebounceJunPro(() {
                  LeooWidgetUtilsJunPro.leooShowPickerWidgetJunPro(
                    context,
                    DateTimePickerAdapter(
                        yearBegin: 2021,
                        yearEnd: DateTime.now().year,
                        value: month != null
                            ? DateTime(int.parse(month!.split('-')[0]),
                                int.parse(month!.split('-')[1]))
                            : null,
                        type: 11),
                    [],
                    S.of(context).detailFilter,
                    (Picker picker, List value) {
                      month = picker.adapter.text.substring(0, 7);
                      status =
                          LeooCommonUtilsJunPro.leooPageStatusLoadingJunPro;
                      LeooCodeUtilsJunPro.onSetState(this);
                      getDetailList();
                    },
                  );
                }),
              ),
            ),
            Expanded(
                child: status !=
                        LeooCommonUtilsJunPro.leooPageStatusSuccessJunPro
                    ? LeooCommonUtilsJunPro.leooCheckPageStatusJunPro(
                        context,
                        status,
                        onTap: () {
                          status =
                              LeooCommonUtilsJunPro.leooPageStatusLoadingJunPro;
                          LeooCodeUtilsJunPro.onSetState(this);
                          getDetailList();
                        },
                      )
                    : SmartRefresher(
                        controller: _refreshController,
                        header: const ClassicHeader(),
                        footer: const ClassicFooter(),
                        enablePullUp: haveMore,
                        onRefresh: () {
                          page = 1;
                          getDetailList();
                        },
                        onLoading: () {
                          page++;
                          getDetailList();
                        },
                        child: ListView.builder(
                            physics: const BouncingScrollPhysics(),
                            itemCount: detailList.length,
                            itemBuilder: (BuildContext itemContext, int index) {
                              LeooDetailEntity item = detailList[index];
                              List<LeooDetailListDetail>? list =
                                  item.listDetail;
                              return Padding(
                                padding:
                                    EdgeInsets.fromLTRB(15.w, 0, 15.w, 15.w),
                                child: Column(
                                  crossAxisAlignment: CrossAxisAlignment.start,
                                  children: [
                                    Text(item.month ?? '',
                                        style: LeooTextJunPro
                                            .leooGetTextStyleJunPro(
                                                20, FontWeight.bold)),
                                    SizedBox(height: 5.w),
                                    Text(
                                        S.of(context).detailIncomeWithdraw(
                                            item.inAmount ?? 0,
                                            item.outAmount ?? 0),
                                        style: LeooTextJunPro
                                            .leooGetTextStyleJunPro(16)),
                                    if (list.isEmpty == false)
                                      Container(
                                        decoration: BoxDecoration(
                                            borderRadius:
                                                BorderRadius.circular(10),
                                            color: LeooColorsJunPro
                                                .leooBtnBgJunPro),
                                        margin: EdgeInsets.only(top: 10.w),
                                        child: ListView.builder(
                                            physics:
                                                const NeverScrollableScrollPhysics(),
                                            itemCount: list.length,
                                            shrinkWrap: true,
                                            itemBuilder:
                                                (BuildContext itemContext,
                                                    int index) {
                                              LeooDetailListDetail item =
                                                  list[index];
                                              return Padding(
                                                padding: EdgeInsets.all(15.w),
                                                child: Row(
                                                  children: [
                                                    LeooCommonUtilsJunPro
                                                        .leooGetLoadWidgetJunPro(
                                                      LeooCodeUtilsJunPro
                                                          .leooGetDetailTypeJunPro(
                                                              item.bizType),
                                                      width: 40.w,
                                                      height: 40.w,
                                                    ),
                                                    SizedBox(width: 10.w),
                                                    Expanded(
                                                      child: Column(
                                                        crossAxisAlignment:
                                                            CrossAxisAlignment
                                                                .start,
                                                        children: [
                                                          Text(
                                                              item.bizTypeName ??
                                                                  '',
                                                              style: LeooTextJunPro
                                                                  .leooGetTextStyleJunPro(
                                                                      20,
                                                                      FontWeight
                                                                          .bold)),
                                                          Text(
                                                              item.bizTime ??
                                                                  '',
                                                              style: LeooTextJunPro
                                                                  .leooGetTextStyleHintJunPro(
                                                                      16)),
                                                        ],
                                                      ),
                                                    ),
                                                    SizedBox(width: 10.w),
                                                    Text(
                                                        '\$${item.amount ?? 0}',
                                                        style: LeooTextJunPro
                                                                .leooGetTextStyleJunPro(
                                                                    20,
                                                                    FontWeight
                                                                        .bold)
                                                            .copyWith(
                                                                color: LeooColorsJunPro
                                                                    .leooOnlineJunPro)),
                                                  ],
                                                ),
                                              );
                                            }),
                                      )
                                  ],
                                ),
                              );
                            }),
                      )),
          ],
        ),
      ),
    );
  }
}
